@@ -1,7 +1,3 @@
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-
 #include "common.h"
 #include "inflate.h"
 #include "crc32.h"
@@ -22,7 +18,7 @@ This package is licensed under the LGPL. You are free to use this library
 in both commercial and non-commercial applications as long as you dynamically
 link to it. If you statically link this library you must also release your
 software under the LGPL. If you need more flexibility in the license email
-me and we can work something out. 
+me and we can work something out.
 
 Michael Kohn <mike@mikekohn.net>
 
@@ -89,7 +85,7 @@ static const unsigned char reverse[256] = {
 10, 138, 74, 202, 42, 170, 106, 234, 26, 154, 90, 218, 58, 186, 122, 250,
 6, 134, 70, 198, 38, 166, 102, 230, 22, 150, 86, 214, 54, 182, 118, 246,
 14, 142, 78, 206, 46, 174, 110, 238, 30, 158, 94, 222, 62, 190, 126, 254,
-1, 129, 65, 193, 33, 161, 97, 225, 17, 145, 81, 209, 49, 177, 113, 241,         
+1, 129, 65, 193, 33, 161, 97, 225, 17, 145, 81, 209, 49, 177, 113, 241,
 9, 137, 73, 201, 41, 169, 105, 233, 25, 153, 89, 217, 57, 185, 121, 249,
 5, 133, 69, 197, 37, 165, 101, 229, 21, 149, 85, 213, 53, 181, 117, 245,
 13, 141, 77, 205, 45, 173, 109, 237, 29, 157, 93, 221, 61, 189, 125, 253,
@@ -205,7 +201,7 @@ static int add_static_codes_to_tree(struct huffman_tree_t *huffman_tree, int cod
       }
 
       x=x>>1;
-    } 
+    }
 
     curr_huffman_leaf->code=start_uncomp_code++;
     start_code++;
@@ -229,7 +225,7 @@ printf("load_fixed_huffman()\n");
 #endif
 
 /*
-  code=0x30;   // 0011 0000 
+  code=0x30;   // 0011 0000
   for (t=0; t<=143; t++)
   {
     huffman->len[t]=8;
@@ -237,7 +233,7 @@ printf("load_fixed_huffman()\n");
     code++;
   }
 
-  code=0x190;  // 1 1001 0000 
+  code=0x190;  // 1 1001 0000
   for (t=144; t<=255; t++)
   {
     huffman->len[t]=9;
@@ -287,7 +283,7 @@ printf("load_fixed_huffman()\n");
 
 static int load_codes(FILE *in, struct bitstream_t *bitstream, int *lengths, int len_size, int count, int *hclen_code_length, int *hclen_code, struct huffman_tree_t *huffman_tree)
 {
-  int r,t,c,x;
+  int r,t,c,x,b;
   int code,curr_code;
   int bl_count[512];
   int next_code[512];
@@ -306,7 +302,11 @@ static int load_codes(FILE *in, struct bitstream_t *bitstream, int *lengths, int
       if (hclen_code_length[t]==0) continue;
       while (bitstream->bitptr<hclen_code_length[t])
       {
-        bitstream->holding=reverse[getc(in)]+(bitstream->holding<<8);
+        b = getc(in);
+        if (b < 0)
+          return -1;
+
+        bitstream->holding=reverse[b]+(bitstream->holding<<8);
         bitstream->bitptr+=8;
       }
 
@@ -347,7 +347,11 @@ static int load_codes(FILE *in, struct bitstream_t *bitstream, int *lengths, int
 
       if (bitstream->bitptr<2)
       {
-        bitstream->holding=reverse[getc(in)]+(bitstream->holding<<8);
+        b = getc(in);
+        if (b < 0)
+          return -1;
+
+        bitstream->holding=reverse[b]+(bitstream->holding<<8);
         bitstream->bitptr+=8;
       }
 
@@ -367,7 +371,11 @@ static int load_codes(FILE *in, struct bitstream_t *bitstream, int *lengths, int
     {
       if (bitstream->bitptr<3)
       {
-        bitstream->holding=reverse[getc(in)]+(bitstream->holding<<8);
+        b = getc(in);
+        if (b < 0)
+          return -1;
+
+        bitstream->holding=reverse[b]+(bitstream->holding<<8);
         bitstream->bitptr+=8;
       }
 
@@ -389,7 +397,11 @@ static int load_codes(FILE *in, struct bitstream_t *bitstream, int *lengths, int
     {
       if (bitstream->bitptr<7)
       {
-        bitstream->holding=reverse[getc(in)]+(bitstream->holding<<8);
+        b = getc(in);
+        if (b < 0)
+          return -1;
+
+        bitstream->holding=reverse[b]+(bitstream->holding<<8);
         bitstream->bitptr+=8;
       }
 
@@ -491,7 +503,7 @@ static int load_codes(FILE *in, struct bitstream_t *bitstream, int *lengths, int
         }
 
         x=x>>1;
-      } 
+      }
 
       huffman_tree[curr_leaf].code=t;
 
@@ -515,11 +527,16 @@ static int load_dynamic_huffman(FILE *in, struct huffman_t *huffman, struct bits
   int next_code[19];
   int code,bits;
   int t;
+  int b;
   int res;
 
   while (bitstream->bitptr<14)
   {
-    bitstream->holding=reverse[getc(in)]+(bitstream->holding<<8);
+    b = getc(in);
+    if (b < 0)
+      return -1;
+
+    bitstream->holding=reverse[b]+(bitstream->holding<<8);
     bitstream->bitptr+=8;
   }
 
@@ -564,7 +581,11 @@ static int load_dynamic_huffman(FILE *in, struct huffman_t *huffman, struct bits
 
     if (bitstream->bitptr<3)
     {
-      bitstream->holding=reverse[getc(in)]+(bitstream->holding<<8);
+      b = getc(in);
+      if (b < 0)
+        return -1;
+
+      bitstream->holding=reverse[b]+(bitstream->holding<<8);
       bitstream->bitptr+=8;
     }
 
@@ -809,7 +830,11 @@ int decompress(FILE *in, struct huffman_t *huffman, struct bitstream_t *bitstrea
       {
         while (bitstream->bitptr<length_extra_bits[code])
         {
-          bitstream->holding+=(getc(in)<<bitstream->bitptr);
+          int x = getc(in);
+          if (x < 0)
+            return -1;
+
+          bitstream->holding+=(x<<bitstream->bitptr);
           bitstream->bitptr+=8;
         }
 
@@ -827,10 +852,14 @@ int decompress(FILE *in, struct huffman_t *huffman, struct bitstream_t *bitstrea
       {
         if (bitstream->bitptr<5)
         {
-          bitstream->holding+=(getc(in)<<bitstream->bitptr);
+          int x = getc(in);
+          if (x < 0)
+            return -1;
+
+          bitstream->holding+=(x<<bitstream->bitptr);
           bitstream->bitptr+=8;
         }
- 
+
         code=(bitstream->holding&0x1f);
         code=reverse[code&255]>>3;
         bitstream->bitptr-=5;
@@ -851,7 +880,11 @@ int decompress(FILE *in, struct huffman_t *huffman, struct bitstream_t *bitstrea
           {
             /* bitstream->holding+=(getc(in)<<bitstream->bitptr); */
             /* bitstream->bitptr+=8; */
-            bitstream->holding=getc(in);
+            int x = getc(in);
+            if (x < 0)
+              return -1;
+
+            bitstream->holding=x;
             bitstream->bitptr=8;
           }
 #ifdef DEBUG
@@ -905,7 +938,11 @@ int decompress(FILE *in, struct huffman_t *huffman, struct bitstream_t *bitstrea
       {
         while (bitstream->bitptr<dist_extra_bits[code])
         {
-          bitstream->holding+=(getc(in)<<bitstream->bitptr);
+          int x = getc(in);
+          if (x < 0)
+            return -1;
+
+          bitstream->holding+=(x<<bitstream->bitptr);
           bitstream->bitptr+=8;
         }
 
@@ -989,15 +1026,15 @@ int libxmp_inflate(FILE *in, FILE *out, uint32 *checksum, int is_zip)
   int comp_method;
   int block_len,bfinal;
   int t;
-  struct huffman_tree_t *huffman_tree_len;
-  struct huffman_tree_t *huffman_tree_dist;
+  struct huffman_tree_t *huffman_tree_len = NULL;
+  struct huffman_tree_t *huffman_tree_dist = NULL;
   struct inflate_data data;
   int res;
 
   memset(&huffman, 0, sizeof(struct huffman_t));
   huffman.checksum=0xffffffff;
 
-  data.huffman_tree_len_static=0;
+  data.huffman_tree_len_static = NULL;
 
   huffman_tree_len=malloc(HUFFMAN_TREE_SIZE * sizeof(struct huffman_tree_t));
   if (huffman_tree_len == NULL)
@@ -1007,7 +1044,7 @@ int libxmp_inflate(FILE *in, FILE *out, uint32 *checksum, int is_zip)
 
   huffman_tree_dist=malloc(HUFFMAN_TREE_SIZE * sizeof(struct huffman_tree_t));
   if (huffman_tree_dist == NULL)
-    goto err2;
+    goto err;
 
   memset(huffman_tree_dist, 0xff, HUFFMAN_TREE_SIZE * sizeof(struct huffman_tree_t));
 
@@ -1018,14 +1055,15 @@ int libxmp_inflate(FILE *in, FILE *out, uint32 *checksum, int is_zip)
 #endif
 
 if (!is_zip) {
-  int x;
-  if ((x = getc(in)) < 0) {
-    goto err3;
+  int x = getc(in);
+  if (x < 0) {
+    goto err;
   }
   CMF=x;
 
-  if ((x = getc(in)) < 0) {
-    goto err3;
+  x = getc(in);
+  if (x < 0) {
+    goto err;
   }
   FLG=x;
 
@@ -1042,7 +1080,7 @@ if (!is_zip) {
   if ((CMF&15)!=8)
   {
     /* printf("Unsupported compression used.\n"); */
-    goto err3;
+    goto err;
   }
 
   if ((FLG&32)!=0)
@@ -1053,7 +1091,7 @@ if (!is_zip) {
   if (((CMF*256+FLG)%31)!=0)
   {
     /* printf("FCHECK fails.\n"); */
-    goto err3;
+    goto err;
   }
 }
 
@@ -1066,7 +1104,12 @@ if (!is_zip) {
   {
     if (bitstream.bitptr<3)
     {
-      bitstream.holding=reverse[getc(in)]+(bitstream.holding<<8);
+      res = getc(in);
+      if (res < 0) {
+        goto err;
+      }
+
+      bitstream.holding=reverse[res]+(bitstream.holding<<8);
       bitstream.bitptr+=8;
     }
 
@@ -1104,7 +1147,7 @@ if (!is_zip) {
       {
         res = getc(in);
         if (res < 0) {
-          goto err3;
+          goto err;
         }
         huffman.window[huffman.window_ptr++]=res;
 
@@ -1115,18 +1158,18 @@ if (!is_zip) {
           huffman.window_ptr=0;
         }
       }
-    } 
+    }
       else
-    if (comp_method==2)	/* Reduced with compression factor 1 */
+    if (comp_method==2)
     {
       /* Fixed Huffman */
-      if (data.huffman_tree_len_static==0) {
+      if (data.huffman_tree_len_static == NULL) {
 	if (load_fixed_huffman(&huffman, &data.huffman_tree_len_static) < 0)
-	  goto err4;
+	  goto err;
       }
 
       if (decompress(in, &huffman, &bitstream, data.huffman_tree_len_static, 0, out, &data) < 0) {
-          goto err4;
+          goto err;
       }
 /*
       free(huffman_tree_len);
@@ -1134,17 +1177,16 @@ if (!is_zip) {
 */
     }
       else
-    if (comp_method==1) /* Shrunk */
+    if (comp_method==1)
     {
-
       /* Dynamic Huffman */
       res = load_dynamic_huffman(in,&huffman,&bitstream,huffman_tree_len,huffman_tree_dist);
       if (res < 0) {
-        goto err4;
+        goto err;
       }
 
       if (decompress(in, &huffman, &bitstream, huffman_tree_len, huffman_tree_dist, out, &data) < 0) {
-        goto err4;
+        goto err;
       }
 
     }
@@ -1182,12 +1224,9 @@ if (!is_zip) {
 
   return 0;
 
- err4:
-  free(data.huffman_tree_len_static);
- err3:
-  free(huffman_tree_dist);
- err2:
-  free(huffman_tree_len);
  err:
+  free(data.huffman_tree_len_static);
+  free(huffman_tree_dist);
+  free(huffman_tree_len);
   return -1;
 }
