@@ -7,11 +7,7 @@
  * to use.
  */
 
-#include <stdio.h>
-#include <string.h>
-#include <unistd.h>
-#include <stdlib.h>
-
+#include "common.h"
 #include "readrle.h"
 
 
