@@ -23,14 +23,13 @@
 
 #define NOMARCH_VER	"1.4"
 
-#include <stdio.h>
-#include <string.h>
-#include <stdlib.h>
 #include <ctype.h>
 #include "common.h"
 #include "depacker.h"
+#if 0
 #include "readrle.h"
 #include "readhuff.h"
+#endif
 #include "readlzw.h"
 
 
