@@ -23,8 +23,6 @@
     Modified for xmp by Claudio Matsuoka, 20120812
  */
 
-#include <stdlib.h>
-#include <string.h>
 #include "common.h"
 #include "depacker.h"
 
@@ -231,7 +229,11 @@ static int make_table(struct LhADecrData *dat, int16 nchar, uint8 bitlen[], int1
 
   /* count */
   for(i = 0; i < nchar; i++)
+  {
+    if(bitlen[i] >= ARRAY_SIZE(count))
+      return -1;
     count[bitlen[i]]++;
+  }
 
   /* calculate first code */
   total = 0;
@@ -325,7 +327,7 @@ static int make_table(struct LhADecrData *dat, int16 nchar, uint8 bitlen[], int1
     }
     start[k] = l;
   }
-  
+
   return 0;
 }
 
@@ -456,7 +458,13 @@ static int decode_c_st1(struct LhADecrData *dat)
   dat->d.st.blocksize--;
   j = dat->d.st.c_table[dat->bitbuf >> 4];
   if(j < NC)
+  {
+    /* Sanity check - 0-length character encoding on the Huffman tree is
+     * invalid and can cause hangs here. */
+    if(dat->d.st.c_len[j] == 0)
+      return -1;
     fillbuf(dat, dat->d.st.c_len[j]);
+  }
   else
   {
     fillbuf(dat, 12);
@@ -1504,14 +1512,17 @@ static int32 LhA_Decrunch(FILE *in, FILE *out, int size, uint32 Method)
           if (dd->count >= size)
             break;
 
+          if(feof(dd->in))
+            break;
+
           c = DecodeC(dd);
           if (c < 0) {
             goto error;
           }
 
-	  if (dd->error)
-	    break;
-	
+          if(dd->error)
+            break;
+
           if(c <= UCHAR_MAX)
           {
             int res = fputc(c, out);
@@ -1532,7 +1543,7 @@ static int32 LhA_Decrunch(FILE *in, FILE *out, int size, uint32 Method)
               int res = fputc(text[i++ & dicsiz], out);
               if (res < 0) {
                 goto error;
-              }        
+              }
               text[dd->loc++] = res;
               dd->loc &= dicsiz;
             }
