@@ -211,19 +211,21 @@ struct lzma_dec {
 	uint16 literal[LITERAL_CODERS_MAX][LITERAL_CODER_SIZE];
 };
 
+enum lzma2_seq {
+	SEQ_CONTROL,
+	SEQ_UNCOMPRESSED_1,
+	SEQ_UNCOMPRESSED_2,
+	SEQ_COMPRESSED_0,
+	SEQ_COMPRESSED_1,
+	SEQ_PROPERTIES,
+	SEQ_LZMA_PREPARE,
+	SEQ_LZMA_RUN,
+	SEQ_COPY
+};
+
 struct lzma2_dec {
 	/* Position in xz_dec_lzma2_run(). */
-	enum lzma2_seq {
-		SEQ_CONTROL,
-		SEQ_UNCOMPRESSED_1,
-		SEQ_UNCOMPRESSED_2,
-		SEQ_COMPRESSED_0,
-		SEQ_COMPRESSED_1,
-		SEQ_PROPERTIES,
-		SEQ_LZMA_PREPARE,
-		SEQ_LZMA_RUN,
-		SEQ_COPY
-	} sequence;
+	enum lzma2_seq sequence;
 
 	/* Next position after decoding the compressed size of the chunk. */
 	enum lzma2_seq next_sequence;
@@ -241,13 +243,13 @@ struct lzma2_dec {
 	 * True if dictionary reset is needed. This is false before
 	 * the first chunk (LZMA or uncompressed).
 	 */
-	bool need_dict_reset;
+	xz_bool need_dict_reset;
 
 	/*
 	 * True if new LZMA properties are needed. This is false
 	 * before the first LZMA chunk.
 	 */
-	bool need_props;
+	xz_bool need_props;
 };
 
 struct xz_dec_lzma2 {
@@ -306,7 +308,7 @@ static void dict_limit(struct dictionary *dict, size_t out_max)
 }
 
 /* Return true if at least one byte can be written into the dictionary. */
-static inline bool dict_has_space(const struct dictionary *dict)
+static inline xz_bool dict_has_space(const struct dictionary *dict)
 {
 	return dict->pos < dict->limit;
 }
@@ -343,13 +345,13 @@ static inline void dict_put(struct dictionary *dict, uint8 byte)
  * invalid, false is returned. On success, true is returned and *len is
  * updated to indicate how many bytes were left to be repeated.
  */
-static bool dict_repeat(struct dictionary *dict, uint32 *len, uint32 dist)
+static xz_bool dict_repeat(struct dictionary *dict, uint32 *len, uint32 dist)
 {
 	size_t back;
 	uint32 left;
 
 	if (dist >= dict->full || dist >= dict->size)
-		return false;
+		return xz_false;
 
 	left = min_t(size_t, dict->limit - dict->pos, *len);
 	*len -= left;
@@ -367,7 +369,7 @@ static bool dict_repeat(struct dictionary *dict, uint32 *len, uint32 dist)
 	if (dict->full < dict->pos)
 		dict->full = dict->pos;
 
-	return true;
+	return xz_true;
 }
 
 /* Copy uncompressed data as is from input to dictionary and output buffers. */
@@ -446,21 +448,21 @@ static void rc_reset(struct rc_dec *rc)
  * Read the first five initial bytes into rc->code if they haven't been
  * read already. (Yes, the first byte gets completely ignored.)
  */
-static bool rc_read_init(struct rc_dec *rc, struct xz_buf *b)
+static xz_bool rc_read_init(struct rc_dec *rc, struct xz_buf *b)
 {
 	while (rc->init_bytes_left > 0) {
 		if (b->in_pos == b->in_size)
-			return false;
+			return xz_false;
 
 		rc->code = (rc->code << 8) + b->in[b->in_pos++];
 		--rc->init_bytes_left;
 	}
 
-	return true;
+	return xz_true;
 }
 
 /* Return true if there may not be enough input for the next decoding loop. */
-static inline bool rc_limit_exceeded(const struct rc_dec *rc)
+static inline xz_bool rc_limit_exceeded(const struct rc_dec *rc)
 {
 	return rc->in_pos > rc->in_limit;
 }
@@ -469,7 +471,7 @@ static inline bool rc_limit_exceeded(const struct rc_dec *rc)
  * Return true if it is possible (from point of view of range decoder) that
  * we have reached the end of the LZMA chunk.
  */
-static inline bool rc_is_finished(const struct rc_dec *rc)
+static inline xz_bool rc_is_finished(const struct rc_dec *rc)
 {
 	return rc->code == 0;
 }
@@ -719,7 +721,7 @@ static void lzma_rep_match(struct xz_dec_lzma2 *s, uint32 pos_state)
 }
 
 /* LZMA decoder core */
-static bool lzma_main(struct xz_dec_lzma2 *s)
+static xz_bool lzma_main(struct xz_dec_lzma2 *s)
 {
 	uint32 pos_state;
 
@@ -747,7 +749,7 @@ static bool lzma_main(struct xz_dec_lzma2 *s)
 				lzma_match(s, pos_state);
 
 			if (!dict_repeat(&s->dict, &s->lzma.len, s->lzma.rep0))
-				return false;
+				return xz_false;
 		}
 	}
 
@@ -757,7 +759,7 @@ static bool lzma_main(struct xz_dec_lzma2 *s)
 	 */
 	rc_normalize(&s->rc);
 
-	return true;
+	return xz_true;
 }
 
 /*
@@ -796,10 +798,10 @@ static void lzma_reset(struct xz_dec_lzma2 *s)
  * from the decoded lp and pb values. On success, the LZMA decoder state is
  * reset and true is returned.
  */
-static bool lzma_props(struct xz_dec_lzma2 *s, uint8 props)
+static xz_bool lzma_props(struct xz_dec_lzma2 *s, uint8 props)
 {
 	if (props > (4 * 5 + 4) * 9 + 8)
-		return false;
+		return xz_false;
 
 	s->lzma.pos_mask = 0;
 	while (props >= 9 * 5) {
@@ -818,13 +820,13 @@ static bool lzma_props(struct xz_dec_lzma2 *s, uint8 props)
 	s->lzma.lc = props;
 
 	if (s->lzma.lc + s->lzma.literal_pos_mask > 4)
-		return false;
+		return xz_false;
 
 	s->lzma.literal_pos_mask = (1 << s->lzma.literal_pos_mask) - 1;
 
 	lzma_reset(s);
 
-	return true;
+	return xz_true;
 }
 
 /*********
@@ -843,7 +845,7 @@ static bool lzma_props(struct xz_dec_lzma2 *s, uint8 props)
  * function. We decode a few bytes from the temporary buffer so that we can
  * continue decoding from the caller-supplied input buffer again.
  */
-static bool lzma2_lzma(struct xz_dec_lzma2 *s, struct xz_buf *b)
+static xz_bool lzma2_lzma(struct xz_dec_lzma2 *s, struct xz_buf *b)
 {
 	size_t in_avail;
 	uint32 tmp;
@@ -866,7 +868,7 @@ static bool lzma2_lzma(struct xz_dec_lzma2 *s, struct xz_buf *b)
 		} else if (s->temp.size + tmp < LZMA_IN_REQUIRED) {
 			s->temp.size += tmp;
 			b->in_pos += tmp;
-			return true;
+			return xz_true;
 		} else {
 			s->rc.in_limit = s->temp.size + tmp - LZMA_IN_REQUIRED;
 		}
@@ -875,7 +877,7 @@ static bool lzma2_lzma(struct xz_dec_lzma2 *s, struct xz_buf *b)
 		s->rc.in_pos = 0;
 
 		if (!lzma_main(s) || s->rc.in_pos > s->temp.size + tmp)
-			return false;
+			return xz_false;
 
 		s->lzma2.compressed -= s->rc.in_pos;
 
@@ -883,7 +885,7 @@ static bool lzma2_lzma(struct xz_dec_lzma2 *s, struct xz_buf *b)
 			s->temp.size -= s->rc.in_pos;
 			memmove(s->temp.buf, s->temp.buf + s->rc.in_pos,
 					s->temp.size);
-			return true;
+			return xz_true;
 		}
 
 		b->in_pos += s->rc.in_pos - s->temp.size;
@@ -901,11 +903,11 @@ static bool lzma2_lzma(struct xz_dec_lzma2 *s, struct xz_buf *b)
 			s->rc.in_limit = b->in_size - LZMA_IN_REQUIRED;
 
 		if (!lzma_main(s))
-			return false;
+			return xz_false;
 
 		in_avail = s->rc.in_pos - b->in_pos;
 		if (in_avail > s->lzma2.compressed)
-			return false;
+			return xz_false;
 
 		s->lzma2.compressed -= in_avail;
 		b->in_pos = s->rc.in_pos;
@@ -921,7 +923,7 @@ static bool lzma2_lzma(struct xz_dec_lzma2 *s, struct xz_buf *b)
 		b->in_pos += in_avail;
 	}
 
-	return true;
+	return xz_true;
 }
 
 /*
@@ -973,8 +975,8 @@ XZ_EXTERN enum xz_ret xz_dec_lzma2_run(struct xz_dec_lzma2 *s,
 				return XZ_STREAM_END;
 
 			if (tmp >= 0xE0 || tmp == 0x01) {
-				s->lzma2.need_props = true;
-				s->lzma2.need_dict_reset = false;
+				s->lzma2.need_props = xz_true;
+				s->lzma2.need_dict_reset = xz_false;
 				dict_reset(&s->dict, b);
 			} else if (s->lzma2.need_dict_reset) {
 				return XZ_DATA_ERROR;
@@ -990,7 +992,7 @@ XZ_EXTERN enum xz_ret xz_dec_lzma2_run(struct xz_dec_lzma2 *s,
 					 * state reset is done at
 					 * SEQ_PROPERTIES.
 					 */
-					s->lzma2.need_props = false;
+					s->lzma2.need_props = xz_false;
 					s->lzma2.next_sequence
 							= SEQ_PROPERTIES;
 
@@ -1155,7 +1157,7 @@ XZ_EXTERN enum xz_ret xz_dec_lzma2_reset(struct xz_dec_lzma2 *s, uint8 props)
 	s->lzma.len = 0;
 
 	s->lzma2.sequence = SEQ_CONTROL;
-	s->lzma2.need_dict_reset = true;
+	s->lzma2.need_dict_reset = xz_true;
 
 	s->temp.size = 0;
 
