@@ -32,6 +32,7 @@
 #define FX_XF_PORTA	0x21
 
 /* Protracker extended effects */
+#define EX_FILTER	0x00
 #define EX_F_PORTA_UP	0x01
 #define EX_F_PORTA_DN	0x02
 #define EX_GLISS	0x03
@@ -39,6 +40,7 @@
 #define EX_FINETUNE	0x05
 #define EX_PATTERN_LOOP	0x06
 #define EX_TREMOLO_WF	0x07
+#define EX_SETPAN	0x08
 #define EX_RETRIG	0x09
 #define EX_F_VSLIDE_UP	0x0a
 #define EX_F_VSLIDE_DN	0x0b
@@ -57,7 +59,7 @@
 #define FX_F_NSLIDE_DN	0x75
 #define FX_F_NSLIDE_UP	0x76
 
-/* Persistent effects -- for 669, FNK and FAR */
+/* Persistent effects -- for FNK and FAR */
 #define FX_PER_PORTA_DN	0x78
 #define FX_PER_PORTA_UP	0x79
 #define FX_PER_TPORTA	0x7a
@@ -66,6 +68,13 @@
 #define FX_PER_VSLD_DN	0x7d
 #define FX_SPEED_CP	0x7e
 #define FX_PER_CANCEL	0x7f
+
+/* 669 frequency based effects */
+#define FX_669_PORTA_UP	0x60
+#define FX_669_PORTA_DN	0x61
+#define FX_669_TPORTA	0x62
+#define FX_669_FINETUNE	0x63
+#define FX_669_VIBRATO	0x64
 #endif
 
 #ifndef LIBXMP_CORE_DISABLE_IT
