@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2016 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2021 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -20,12 +20,10 @@
  * THE SOFTWARE.
  */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <limits.h>
 #include <errno.h>
 #include "common.h"
 #include "hio.h"
+#include "callbackio.h"
 #include "mdataio.h"
 
 static long get_size(FILE *f)
@@ -50,159 +48,200 @@ static long get_size(FILE *f)
 int8 hio_read8s(HIO_HANDLE *h)
 {
 	int err;
-	int8 ret = 0;
+	int8 ret;
 
 	switch (HIO_HANDLE_TYPE(h)) {
 	case HIO_HANDLE_TYPE_FILE:
 		ret = read8s(h->handle.file, &err);
-		if (err != 0) {
-			h->error = err;
-		}
 		break;
 	case HIO_HANDLE_TYPE_MEMORY:
-		ret = mread8s(h->handle.mem);
+		ret = mread8s(h->handle.mem, &err);
 		break;
+	case HIO_HANDLE_TYPE_CBFILE:
+		ret = cbread8s(h->handle.cbfile, &err);
+		break;
+	default:
+		return 0;
 	}
 
+	if (err != 0) {
+		h->error = err;
+	}
 	return ret;
 }
 
 uint8 hio_read8(HIO_HANDLE *h)
 {
 	int err;
-	uint8 ret = 0;
+	uint8 ret;
 
 	switch (HIO_HANDLE_TYPE(h)) {
 	case HIO_HANDLE_TYPE_FILE:
 		ret = read8(h->handle.file, &err);
-		if (err != 0) {
-			h->error = err;
-		}
 		break;
 	case HIO_HANDLE_TYPE_MEMORY:
-		ret = mread8(h->handle.mem);
+		ret = mread8(h->handle.mem, &err);
 		break;
+	case HIO_HANDLE_TYPE_CBFILE:
+		ret = cbread8(h->handle.cbfile, &err);
+		break;
+	default:
+		return 0;
 	}
 
+	if (err != 0) {
+		h->error = err;
+	}
 	return ret;
 }
 
 uint16 hio_read16l(HIO_HANDLE *h)
 {
 	int err;
-	uint16 ret = 0;
+	uint16 ret;
 
 	switch (HIO_HANDLE_TYPE(h)) {
 	case HIO_HANDLE_TYPE_FILE:
 		ret = read16l(h->handle.file, &err);
-		if (err != 0) {
-			h->error = err;
-		}
 		break;
 	case HIO_HANDLE_TYPE_MEMORY:
-		ret = mread16l(h->handle.mem);
+		ret = mread16l(h->handle.mem, &err);
 		break;
+	case HIO_HANDLE_TYPE_CBFILE:
+		ret = cbread16l(h->handle.cbfile, &err);
+		break;
+	default:
+		return 0;
 	}
 
+	if (err != 0) {
+		h->error = err;
+	}
 	return ret;
 }
 
 uint16 hio_read16b(HIO_HANDLE *h)
 {
 	int err;
-	uint16 ret = 0;
+	uint16 ret;
 
 	switch (HIO_HANDLE_TYPE(h)) {
 	case HIO_HANDLE_TYPE_FILE:
 		ret = read16b(h->handle.file, &err);
-		if (err != 0) {
-			h->error = err;
-		}
 		break;
 	case HIO_HANDLE_TYPE_MEMORY:
-		ret = mread16b(h->handle.mem);
+		ret = mread16b(h->handle.mem, &err);
 		break;
+	case HIO_HANDLE_TYPE_CBFILE:
+		ret = cbread16b(h->handle.cbfile, &err);
+		break;
+	default:
+		return 0;
 	}
 
+	if (err != 0) {
+		h->error = err;
+	}
 	return ret;
 }
 
 uint32 hio_read24l(HIO_HANDLE *h)
 {
 	int err;
-	uint32 ret = 0;
+	uint32 ret;
 
 	switch (HIO_HANDLE_TYPE(h)) {
 	case HIO_HANDLE_TYPE_FILE:
-		ret = read24l(h->handle.file, &err); 
-		if (err != 0) {
-			h->error = err;
-		}
+		ret = read24l(h->handle.file, &err);
 		break;
 	case HIO_HANDLE_TYPE_MEMORY:
-		ret = mread24l(h->handle.mem); 
+		ret = mread24l(h->handle.mem, &err);
 		break;
+	case HIO_HANDLE_TYPE_CBFILE:
+		ret = cbread24l(h->handle.cbfile, &err);
+		break;
+	default:
+		return 0;
 	}
 
+	if (err != 0) {
+		h->error = err;
+	}
 	return ret;
 }
 
 uint32 hio_read24b(HIO_HANDLE *h)
 {
 	int err;
-	uint32 ret = 0;
+	uint32 ret;
 
 	switch (HIO_HANDLE_TYPE(h)) {
 	case HIO_HANDLE_TYPE_FILE:
 		ret = read24b(h->handle.file, &err);
-		if (err != 0) {
-			h->error = err;
-		}
 		break;
 	case HIO_HANDLE_TYPE_MEMORY:
-		ret = mread24b(h->handle.mem);
+		ret = mread24b(h->handle.mem, &err);
 		break;
+	case HIO_HANDLE_TYPE_CBFILE:
+		ret = cbread24b(h->handle.cbfile, &err);
+		break;
+	default:
+		return 0;
 	}
 
+	if (err != 0) {
+		h->error = err;
+	}
 	return ret;
 }
 
 uint32 hio_read32l(HIO_HANDLE *h)
 {
 	int err;
-	uint32 ret = 0;
+	uint32 ret;
 
 	switch (HIO_HANDLE_TYPE(h)) {
 	case HIO_HANDLE_TYPE_FILE:
 		ret = read32l(h->handle.file, &err);
-		if (err != 0) {
-			h->error = err;
-		}
 		break;
 	case HIO_HANDLE_TYPE_MEMORY:
-		ret = mread32l(h->handle.mem);
+		ret = mread32l(h->handle.mem, &err);
 		break;
+	case HIO_HANDLE_TYPE_CBFILE:
+		ret = cbread32l(h->handle.cbfile, &err);
+		break;
+	default:
+		return 0;
 	}
 
+	if (err != 0) {
+		h->error = err;
+	}
 	return ret;
 }
 
 uint32 hio_read32b(HIO_HANDLE *h)
 {
 	int err;
-	uint32 ret = 0;
+	uint32 ret;
 
 	switch (HIO_HANDLE_TYPE(h)) {
 	case HIO_HANDLE_TYPE_FILE:
 		ret = read32b(h->handle.file, &err);
-		if (err != 0) {
-			h->error = err;
-		}
 		break;
 	case HIO_HANDLE_TYPE_MEMORY:
-		ret = mread32b(h->handle.mem);
+		ret = mread32b(h->handle.mem, &err);
+		break;
+	case HIO_HANDLE_TYPE_CBFILE:
+		ret = cbread32b(h->handle.cbfile, &err);
+		break;
+	default:
+		return 0;
 	}
 
+	if (err != 0) {
+		h->error = err;
+	}
 	return ret;
 }
 
@@ -224,7 +263,13 @@ size_t hio_read(void *buf, size_t size, size_t num, HIO_HANDLE *h)
 	case HIO_HANDLE_TYPE_MEMORY:
 		ret = mread(buf, size, num, h->handle.mem);
 		if (ret != num) {
-			h->error = errno;
+			h->error = EOF;
+		}
+		break;
+	case HIO_HANDLE_TYPE_CBFILE:
+		ret = cbread(buf, size, num, h->handle.cbfile);
+		if (ret != num) {
+			h->error = EOF;
 		}
 		break;
 	}
@@ -246,7 +291,13 @@ int hio_seek(HIO_HANDLE *h, long offset, int whence)
 	case HIO_HANDLE_TYPE_MEMORY:
 		ret = mseek(h->handle.mem, offset, whence);
 		if (ret < 0) {
-			h->error = errno;
+			h->error = EINVAL;
+		}
+		break;
+	case HIO_HANDLE_TYPE_CBFILE:
+		ret = cbseek(h->handle.cbfile, offset, whence);
+		if (ret < 0) {
+			h->error = EINVAL;
 		}
 		break;
 	}
@@ -268,7 +319,14 @@ long hio_tell(HIO_HANDLE *h)
 	case HIO_HANDLE_TYPE_MEMORY:
 		ret = mtell(h->handle.mem);
 		if (ret < 0) {
-			h->error = errno;
+		/* should _not_ happen! */
+			h->error = EINVAL;
+		}
+		break;
+	case HIO_HANDLE_TYPE_CBFILE:
+		ret = cbtell(h->handle.cbfile);
+		if (ret < 0) {
+			h->error = EINVAL;
 		}
 		break;
 	}
@@ -283,9 +341,10 @@ int hio_eof(HIO_HANDLE *h)
 		return feof(h->handle.file);
 	case HIO_HANDLE_TYPE_MEMORY:
 		return meof(h->handle.mem);
-	default:
-		return EOF;
+	case HIO_HANDLE_TYPE_CBFILE:
+		return cbeof(h->handle.cbfile);
 	}
+	return EOF;
 }
 
 int hio_error(HIO_HANDLE *h)
@@ -295,15 +354,14 @@ int hio_error(HIO_HANDLE *h)
 	return error;
 }
 
-HIO_HANDLE *hio_open(void *path, char *mode)
+HIO_HANDLE *hio_open(const char *path, const char *mode)
 {
 	HIO_HANDLE *h;
 
-	h = (HIO_HANDLE *)malloc(sizeof (HIO_HANDLE));
+	h = (HIO_HANDLE *)calloc(1, sizeof (HIO_HANDLE));
 	if (h == NULL)
 		goto err;
-	
-	h->error = 0;
+
 	h->type = HIO_HANDLE_TYPE_FILE;
 	h->handle.file = fopen(path, mode);
 	if (h->handle.file == NULL)
@@ -323,15 +381,15 @@ HIO_HANDLE *hio_open(void *path, char *mode)
 	return NULL;
 }
 
-HIO_HANDLE *hio_open_mem(void *ptr, long size)
+HIO_HANDLE *hio_open_mem(const void *ptr, long size)
 {
 	HIO_HANDLE *h;
 
-	h = (HIO_HANDLE *)malloc(sizeof (HIO_HANDLE));
+	if (size <= 0) return NULL;
+	h = (HIO_HANDLE *)calloc(1, sizeof (HIO_HANDLE));
 	if (h == NULL)
 		return NULL;
-	
-	h->error = 0;
+
 	h->type = HIO_HANDLE_TYPE_MEMORY;
 	h->handle.mem = mopen(ptr, size);
 	h->size = size;
@@ -343,31 +401,72 @@ HIO_HANDLE *hio_open_file(FILE *f)
 {
 	HIO_HANDLE *h;
 
-	h = (HIO_HANDLE *)malloc(sizeof (HIO_HANDLE));
+	h = (HIO_HANDLE *)calloc(1, sizeof (HIO_HANDLE));
 	if (h == NULL)
 		return NULL;
-	
-	h->error = 0;
-	h->type = HIO_HANDLE_TYPE_FILE;
-	h->handle.file = f /*fdopen(fileno(f), "rb")*/;
-	h->size = get_size(f);
 
+	h->noclose = 1;
+	h->type = HIO_HANDLE_TYPE_FILE;
+	h->handle.file = f;
+	h->size = get_size(f);
+	if (h->size < 0) {
+		free(h);
+		return NULL;
+	}
+
+	return h;
+}
+
+HIO_HANDLE *hio_open_file2(FILE *f)
+{
+	HIO_HANDLE *h = hio_open_file(f);
+	if (h != NULL) {
+		h->noclose = 0;
+	}
+	else {
+		fclose(f);
+	}
+	return h;
+}
+
+HIO_HANDLE *hio_open_callbacks(void *priv, struct xmp_callbacks callbacks)
+{
+	HIO_HANDLE *h;
+	CBFILE *f = cbopen(priv, callbacks);
+	if (!f)
+		return NULL;
+
+	h = (HIO_HANDLE *)calloc(1, sizeof(HIO_HANDLE));
+	if (h == NULL) {
+		cbclose(f);
+		return NULL;
+	}
+
+	h->type = HIO_HANDLE_TYPE_CBFILE;
+	h->handle.cbfile = f;
+	h->size = cbfilelength(f);
+	if (h->size < 0) {
+		cbclose(f);
+		free(h);
+		return NULL;
+	}
 	return h;
 }
 
 int hio_close(HIO_HANDLE *h)
 {
-	int ret;
+	int ret = -1;
 
 	switch (HIO_HANDLE_TYPE(h)) {
 	case HIO_HANDLE_TYPE_FILE:
-		ret = fclose(h->handle.file);
+		ret = (h->noclose)? 0 : fclose(h->handle.file);
 		break;
 	case HIO_HANDLE_TYPE_MEMORY:
 		ret = mclose(h->handle.mem);
 		break;
-	default:
-		ret = -1;
+	case HIO_HANDLE_TYPE_CBFILE:
+		ret = cbclose(h->handle.cbfile);
+		break;
 	}
 
 	free(h);
