@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2016 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2021 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -20,214 +20,42 @@
  * THE SOFTWARE.
  */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <ctype.h>
+#include <sys/stat.h>
 #include <errno.h>
-#ifdef __native_client__
-#include <sys/syslimits.h>
-#else
-#include <limits.h>
-#endif
 
 #include "format.h"
 #include "list.h"
 #include "hio.h"
 #include "tempfile.h"
+#include "loaders/loader.h"
+
+#ifndef LIBXMP_NO_DEPACKERS
+#include "depackers/depacker.h"
+#endif
 
 #ifndef LIBXMP_CORE_PLAYER
-#if !defined(HAVE_POPEN) && defined(WIN32)
-#include "win32/ptpopen.h"
-#endif
 #include "md5.h"
 #include "extras.h"
 #endif
 
 
-extern struct format_loader *format_loader[];
+extern struct format_loader *format_loaders[];
 
 void libxmp_load_prologue(struct context_data *);
 void libxmp_load_epilogue(struct context_data *);
 int  libxmp_prepare_scan(struct context_data *);
 
 #ifndef LIBXMP_CORE_PLAYER
-
-#include "depacker.h"
-
-static struct depacker *depacker_list[] = {
-#if defined __AMIGA__ && !defined __AROS__
-	&libxmp_depacker_xfd,
-#endif
-	&libxmp_depacker_zip,
-	&libxmp_depacker_lha,
-	&libxmp_depacker_gzip,
-	&libxmp_depacker_bzip2,
-	&libxmp_depacker_xz,
-	&libxmp_depacker_compress,
-	&libxmp_depacker_pp,
-	&libxmp_depacker_sqsh,
-	&libxmp_depacker_arcfs,
-	&libxmp_depacker_mmcmp,
-	&libxmp_depacker_muse,
-	&libxmp_depacker_lzx,
-	&libxmp_depacker_s404,
-	&libxmp_depacker_arc,
-	NULL
-};
-
-int test_oxm		(FILE *);
-
 #define BUFLEN 16384
-
-static int execute_command(char *cmd, char *filename, FILE *t)
-{
-	char line[1024], buf[BUFLEN];
-	FILE *p;
-	int n;
-
-	snprintf(line, 1024, cmd, filename);
-
-#ifdef WIN32
-	/* Note: The _popen function returns an invalid file opaque, if
-	 * used in a Windows program, that will cause the program to hang
-	 * indefinitely. _popen works properly in a Console application.
-	 * To create a Windows application that redirects input and output,
-	 * read the section "Creating a Child Process with Redirected Input
-	 * and Output" in the Win32 SDK. -- Mirko 
-	 */
-	p = popen(line, "rb");
-#else
-	/* Linux popen fails with "rb" */
-	p = popen(line, "r");
 #endif
 
-	if (p == NULL) {
-	    return -1;
-	}
-
-	while ((n = fread(buf, 1, BUFLEN, p)) > 0) {
-	    fwrite(buf, 1, n, t);
-	}
-
-	pclose (p);
-
-	return 0;
-}
-
-static int decrunch(HIO_HANDLE **h, char *filename, char **temp)
-{
-	unsigned char b[1024];
-	char *cmd;
-	FILE *f, *t;
-	int res;
-	int headersize;
-	int i;
-	struct depacker *depacker = NULL;
-
-	cmd = NULL;
-	res = 0;
-	*temp = NULL;
-	f = (*h)->handle.file;
-
-	headersize = fread(b, 1, 1024, f);
-	if (headersize < 100) {	/* minimum valid file size */
-		return 0;
-	}
-
-	/* Check built-in depackers */
-	for (i = 0; depacker_list[i] != NULL; i++) {
-		if (depacker_list[i]->test(b)) {
-			depacker = depacker_list[i];
-			D_(D_INFO "Use depacker %d", i);
-			break;
-		}
-	}
-
-	/* Check external commands */
-	if (depacker == NULL) {
-		if (b[0] == 'M' && b[1] == 'O' && b[2] == '3') {
-			/* MO3 */
-			D_(D_INFO "mo3");
-			cmd = "unmo3 -s \"%s\" STDOUT";
-		} else if (memcmp(b, "Rar", 3) == 0) {
-			/* rar */
-			D_(D_INFO "rar");
-			cmd = "unrar p -inul -xreadme -x*.diz -x*.nfo -x*.txt "
-			    "-x*.exe -x*.com \"%s\"";
-		} else if (test_oxm(f) == 0) {
-			/* oggmod */
-			D_(D_INFO "oggmod");
-			depacker = &libxmp_depacker_oxm;
-		}
-	}
-
-	if (fseek(f, 0, SEEK_SET) < 0) {
-		goto err;
-	}
-
-	if (depacker == NULL && cmd == NULL) {
-		D_(D_INFO "Not packed");
-		return 0;
-	}
-
-#if defined __ANDROID__ || defined __native_client__
-	/* Don't use external helpers in android */
-	if (cmd) {
-		return 0;
-	}
-#endif
-
-	D_(D_WARN "Depacking file... ");
-
-	if ((t = make_temp_file(temp)) == NULL) {
-		goto err;
-	}
-
-	/* Depack file */
-	if (cmd) {
-		D_(D_INFO "External depacker: %s", cmd);
-		if (execute_command(cmd, filename, t) < 0) {
-			D_(D_CRIT "failed");
-			goto err2;
-		}
-	} else if (depacker) {
-		D_(D_INFO "Internal depacker");
-		if (depacker->depack(f, t) < 0) {
-			D_(D_CRIT "failed");
-			goto err2;
-		}
-	}
-
-	D_(D_INFO "done");
-
-	if (fseek(t, 0, SEEK_SET) < 0) {
-		D_(D_CRIT "fseek error");
-		goto err2;
-	}
-
-	hio_close(*h);
-	*h = hio_open_file(t);
-
-	return res;
-
-    err2:
-	fclose(t);
-    err:
-	return -1;
-}
-
+#ifndef LIBXMP_CORE_PLAYER
 static void set_md5sum(HIO_HANDLE *f, unsigned char *digest)
 {
 	unsigned char buf[BUFLEN];
 	MD5_CTX ctx;
 	int bytes_read;
 
-	if (hio_size(f) <= 0) {
-		memset(digest, 0, 16);
-		return;
-	}
-
 	hio_seek(f, 0, SEEK_SET);
 
 	MD5Init(&ctx);
@@ -237,12 +65,13 @@ static void set_md5sum(HIO_HANDLE *f, unsigned char *digest)
 	MD5Final(digest, &ctx);
 }
 
-static char *get_dirname(char *name)
+static char *get_dirname(const char *name)
 {
-	char *div, *dirname;
-	int len;
+	char *dirname;
+	const char *div;
+	ptrdiff_t len;
 
-	if ((div = strrchr(name, '/'))) {
+	if ((div = strrchr(name, '/')) != NULL) {
 		len = div - name + 1;
 		dirname = malloc(len + 1);
 		if (dirname != NULL) {
@@ -256,11 +85,12 @@ static char *get_dirname(char *name)
 	return dirname;
 }
 
-static char *get_basename(char *name)
+static char *get_basename(const char *name)
 {
-	char *div, *basename;
+	const char *div;
+	char *basename;
 
-	if ((div = strrchr(name, '/'))) {
+	if ((div = strrchr(name, '/')) != NULL) {
 		basename = strdup(div + 1);
 	} else {
 		basename = strdup(name);
@@ -270,32 +100,65 @@ static char *get_basename(char *name)
 }
 #endif /* LIBXMP_CORE_PLAYER */
 
-int xmp_test_module(char *path, struct xmp_test_info *info)
+static int test_module(struct xmp_test_info *info, HIO_HANDLE *h)
+{
+	char buf[XMP_NAME_SIZE];
+	int i;
+
+	if (info != NULL) {
+		*info->name = 0;	/* reset name prior to testing */
+		*info->type = 0;	/* reset type prior to testing */
+	}
+
+	for (i = 0; format_loaders[i] != NULL; i++) {
+		hio_seek(h, 0, SEEK_SET);
+		if (format_loaders[i]->test(h, buf, 0) == 0) {
+			int is_prowizard = 0;
+
+#if !defined(LIBXMP_CORE_PLAYER) && !defined(LIBXMP_NO_PROWIZARD)
+			if (strcmp(format_loaders[i]->name, "prowizard") == 0) {
+				hio_seek(h, 0, SEEK_SET);
+				pw_test_format(h, buf, 0, info);
+				is_prowizard = 1;
+			}
+#endif
+
+			if (info != NULL && !is_prowizard) {
+				strncpy(info->name, buf, XMP_NAME_SIZE - 1);
+				info->name[XMP_NAME_SIZE - 1] = '\0';
+
+				strncpy(info->type, format_loaders[i]->name,
+							XMP_NAME_SIZE - 1);
+				info->type[XMP_NAME_SIZE - 1] = '\0';
+			}
+			return 0;
+		}
+	}
+	return -XMP_ERROR_FORMAT;
+}
+
+int xmp_test_module(const char *path, struct xmp_test_info *info)
 {
 	HIO_HANDLE *h;
 	struct stat st;
-	char buf[XMP_NAME_SIZE];
-	int i;
-	int ret = -XMP_ERROR_FORMAT;
-#ifndef LIBXMP_CORE_PLAYER
+	int ret;
+#ifndef LIBXMP_NO_DEPACKERS
 	char *temp = NULL;
 #endif
 
 	if (stat(path, &st) < 0)
 		return -XMP_ERROR_SYSTEM;
 
-#ifndef _MSC_VER
 	if (S_ISDIR(st.st_mode)) {
 		errno = EISDIR;
 		return -XMP_ERROR_SYSTEM;
 	}
-#endif
 
 	if ((h = hio_open(path, "rb")) == NULL)
 		return -XMP_ERROR_SYSTEM;
 
-#ifndef LIBXMP_CORE_PLAYER
-	if (decrunch(&h, path, &temp) < 0) {
+#ifndef LIBXMP_NO_DEPACKERS
+	if (libxmp_decrunch(&h, path, &temp) < 0) {
 		ret = -XMP_ERROR_DEPACK;
 		goto err;
 	}
@@ -307,40 +170,9 @@ int xmp_test_module(char *path, struct xmp_test_info *info)
 	}
 #endif
 
-	if (info != NULL) {
-		*info->name = 0;	/* reset name prior to testing */
-		*info->type = 0;	/* reset type prior to testing */
-	}
+	ret = test_module(info, h);
 
-	for (i = 0; format_loader[i] != NULL; i++) {
-		hio_seek(h, 0, SEEK_SET);
-		if (format_loader[i]->test(h, buf, 0) == 0) {
-			int is_prowizard = 0;
-
-#ifndef LIBXMP_CORE_PLAYER
-			if (strcmp(format_loader[i]->name, "prowizard") == 0) {
-				hio_seek(h, 0, SEEK_SET);
-				pw_test_format(h, buf, 0, info);
-				is_prowizard = 1;
-			}
-#endif
-
-			fclose(h->handle.file);
-
-#ifndef LIBXMP_CORE_PLAYER
-			unlink_temp_file(temp);
-#endif
-
-			if (info != NULL && !is_prowizard) {
-				strncpy(info->name, buf, XMP_NAME_SIZE - 1);
-				strncpy(info->type, format_loader[i]->name,
-							XMP_NAME_SIZE - 1);
-			}
-			return 0;
-		}
-	}
-
-#ifndef LIBXMP_CORE_PLAYER
+#ifndef LIBXMP_NO_DEPACKERS
     err:
 	hio_close(h);
 	unlink_temp_file(temp);
@@ -350,6 +182,75 @@ int xmp_test_module(char *path, struct xmp_test_info *info)
 	return ret;
 }
 
+int xmp_test_module_from_memory(const void *mem, long size, struct xmp_test_info *info)
+{
+	HIO_HANDLE *h;
+	int ret;
+
+	if (size <= 0) {
+		return -XMP_ERROR_INVALID;
+	}
+
+	if ((h = hio_open_mem(mem, size)) == NULL)
+		return -XMP_ERROR_SYSTEM;
+
+	ret = test_module(info, h);
+
+	hio_close(h);
+	return ret;
+}
+
+int xmp_test_module_from_file(void *file, struct xmp_test_info *info)
+{
+	HIO_HANDLE *h;
+	int ret;
+#ifndef LIBXMP_NO_DEPACKERS
+	char *temp = NULL;
+#endif
+
+	if ((h = hio_open_file((FILE *)file)) == NULL)
+		return -XMP_ERROR_SYSTEM;
+
+#ifndef LIBXMP_NO_DEPACKERS
+	if (libxmp_decrunch(&h, NULL, &temp) < 0) {
+		ret = -XMP_ERROR_DEPACK;
+		goto err;
+	}
+
+	/* get size after decrunch */
+	if (hio_size(h) < 256) {	/* set minimum valid module size */
+		ret = -XMP_ERROR_FORMAT;
+		goto err;
+	}
+#endif
+
+	ret = test_module(info, h);
+
+#ifndef LIBXMP_NO_DEPACKERS
+    err:
+	hio_close(h);
+	unlink_temp_file(temp);
+#else
+	hio_close(h);
+#endif
+	return ret;
+}
+
+int xmp_test_module_from_callbacks(void *priv, struct xmp_callbacks callbacks,
+				struct xmp_test_info *info)
+{
+	HIO_HANDLE *h;
+	int ret;
+
+	if ((h = hio_open_callbacks(priv, callbacks)) == NULL)
+		return -XMP_ERROR_SYSTEM;
+
+	ret = test_module(info, h);
+
+	hio_close(h);
+	return ret;
+}
+
 static int load_module(xmp_context opaque, HIO_HANDLE *h)
 {
 	struct context_data *ctx = (struct context_data *)opaque;
@@ -362,19 +263,16 @@ static int load_module(xmp_context opaque, HIO_HANDLE *h)
 
 	D_(D_WARN "load");
 	test_result = load_result = -1;
-	for (i = 0; format_loader[i] != NULL; i++) {
+	for (i = 0; format_loaders[i] != NULL; i++) {
 		hio_seek(h, 0, SEEK_SET);
+		hio_error(h); /* reset error flag */
 
-		if (hio_error(h)) {
-			/* reset error flag */
-		}
-
-		D_(D_WARN "test %s", format_loader[i]->name);
-		test_result = format_loader[i]->test(h, NULL, 0);
+		D_(D_WARN "test %s", format_loaders[i]->name);
+		test_result = format_loaders[i]->test(h, NULL, 0);
 		if (test_result == 0) {
 			hio_seek(h, 0, SEEK_SET);
-			D_(D_WARN "load format: %s", format_loader[i]->name);
-			load_result = format_loader[i]->loader(m, h, 0);
+			D_(D_WARN "load format: %s", format_loaders[i]->name);
+			load_result = format_loaders[i]->loader(m, h, 0);
 			break;
 		}
 	}
@@ -385,8 +283,7 @@ static int load_module(xmp_context opaque, HIO_HANDLE *h)
 #endif
 
 	if (test_result < 0) {
-		free(m->basename);
-		free(m->dirname);
+		xmp_release_module(opaque);
 		return -XMP_ERROR_FORMAT;
 	}
 
@@ -441,7 +338,11 @@ static int load_module(xmp_context opaque, HIO_HANDLE *h)
 		return ret;
 	}
 
-	libxmp_scan_sequences(ctx);
+	ret = libxmp_scan_sequences(ctx);
+	if (ret < 0) {
+		xmp_release_module(opaque);
+		return -XMP_ERROR_LOAD;
+	}
 
 	ctx->state = XMP_STATE_LOADED;
 
@@ -452,12 +353,14 @@ static int load_module(xmp_context opaque, HIO_HANDLE *h)
 	return -XMP_ERROR_LOAD;
 }
 
-int xmp_load_module(xmp_context opaque, char *path)
+int xmp_load_module(xmp_context opaque, const char *path)
 {
 	struct context_data *ctx = (struct context_data *)opaque;
 #ifndef LIBXMP_CORE_PLAYER
 	struct module_data *m = &ctx->m;
 	long size;
+#endif
+#ifndef LIBXMP_NO_DEPACKERS
 	char *temp_name;
 #endif
 	HIO_HANDLE *h;
@@ -470,24 +373,24 @@ int xmp_load_module(xmp_context opaque, char *path)
 		return -XMP_ERROR_SYSTEM;
 	}
 
-#ifndef _MSC_VER
 	if (S_ISDIR(st.st_mode)) {
 		errno = EISDIR;
 		return -XMP_ERROR_SYSTEM;
 	}
-#endif
 
 	if ((h = hio_open(path, "rb")) == NULL) {
 		return -XMP_ERROR_SYSTEM;
 	}
 
-#ifndef LIBXMP_CORE_PLAYER
+#ifndef LIBXMP_NO_DEPACKERS
 	D_(D_INFO "decrunch");
-	if (decrunch(&h, path, &temp_name) < 0) {
+	if (libxmp_decrunch(&h, path, &temp_name) < 0) {
 		ret = -XMP_ERROR_DEPACK;
 		goto err;
 	}
+#endif
 
+#ifndef LIBXMP_CORE_PLAYER
 	size = hio_size(h);
 	if (size < 256) {		/* get size after decrunch */
 		ret = -XMP_ERROR_FORMAT;
@@ -513,12 +416,16 @@ int xmp_load_module(xmp_context opaque, char *path)
 
 	m->filename = path;	/* For ALM, SSMT, etc */
 	m->size = size;
+#else
+	ctx->m.filename = NULL;
+	ctx->m.dirname = NULL;
+	ctx->m.basename = NULL;
 #endif
 
 	ret = load_module(opaque, h);
 	hio_close(h);
 
-#ifndef LIBXMP_CORE_PLAYER
+#ifndef LIBXMP_NO_DEPACKERS
 	unlink_temp_file(temp_name);
 #endif
 
@@ -527,21 +434,23 @@ int xmp_load_module(xmp_context opaque, char *path)
 #ifndef LIBXMP_CORE_PLAYER
     err:
 	hio_close(h);
+#ifndef LIBXMP_NO_DEPACKERS
 	unlink_temp_file(temp_name);
+#endif
 	return ret;
 #endif
 }
 
-int xmp_load_module_from_memory(xmp_context opaque, void *mem, long size)
+int xmp_load_module_from_memory(xmp_context opaque, const void *mem, long size)
 {
 	struct context_data *ctx = (struct context_data *)opaque;
 	struct module_data *m = &ctx->m;
 	HIO_HANDLE *h;
 	int ret;
 
-	/* Use size < 0 for unknown/undetermined size */
-	if (size == 0)
-		size--;
+	if (size <= 0) {
+		return -XMP_ERROR_INVALID;
+	}
 
 	if ((h = hio_open_mem(mem, size)) == NULL)
 		return -XMP_ERROR_SYSTEM;
@@ -566,10 +475,35 @@ int xmp_load_module_from_file(xmp_context opaque, void *file, long size)
 	struct context_data *ctx = (struct context_data *)opaque;
 	struct module_data *m = &ctx->m;
 	HIO_HANDLE *h;
-	FILE *f = fdopen(fileno((FILE *)file), "rb");
 	int ret;
 
-	if ((h = hio_open_file(f)) == NULL)
+	if ((h = hio_open_file((FILE *)file)) == NULL)
+		return -XMP_ERROR_SYSTEM;
+
+	if (ctx->state > XMP_STATE_UNLOADED)
+		xmp_release_module(opaque);
+
+	m->filename = NULL;
+	m->basename = NULL;
+	m->dirname = NULL;
+	m->size = hio_size(h);
+
+	ret = load_module(opaque, h);
+
+	hio_close(h);
+
+	return ret;
+}
+
+int xmp_load_module_from_callbacks(xmp_context opaque, void *priv,
+				struct xmp_callbacks callbacks)
+{
+	struct context_data *ctx = (struct context_data *)opaque;
+	struct module_data *m = &ctx->m;
+	HIO_HANDLE *h;
+	int ret;
+
+	if ((h = hio_open_callbacks(priv, callbacks)) == NULL)
 		return -XMP_ERROR_SYSTEM;
 
 	if (ctx->state > XMP_STATE_UNLOADED)
@@ -616,6 +550,7 @@ void xmp_release_module(xmp_context opaque)
 			free(mod->xxt[i]);
 		}
 		free(mod->xxt);
+		mod->xxt = NULL;
 	}
 
 	if (mod->xxp != NULL) {
@@ -623,6 +558,7 @@ void xmp_release_module(xmp_context opaque)
 			free(mod->xxp[i]);
 		}
 		free(mod->xxp);
+		mod->xxp = NULL;
 	}
 
 	if (mod->xxi != NULL) {
@@ -631,40 +567,40 @@ void xmp_release_module(xmp_context opaque)
 			free(mod->xxi[i].extra);
 		}
 		free(mod->xxi);
+		mod->xxi = NULL;
 	}
 
 	if (mod->xxs != NULL) {
 		for (i = 0; i < mod->smp; i++) {
-			if (mod->xxs[i].data != NULL) {
-				free(mod->xxs[i].data - 4);
-			}
+			libxmp_free_sample(&mod->xxs[i]);
 		}
 		free(mod->xxs);
-		free(m->xtra);
+		mod->xxs = NULL;
 	}
 
+	free(m->xtra);
+	m->xtra = NULL;
+
 #ifndef LIBXMP_CORE_DISABLE_IT
 	if (m->xsmp != NULL) {
 		for (i = 0; i < mod->smp; i++) {
-			if (m->xsmp[i].data != NULL) {
-				free(m->xsmp[i].data - 4);
-			}
+			libxmp_free_sample(&m->xsmp[i]);
 		}
 		free(m->xsmp);
+		m->xsmp = NULL;
 	}
 #endif
 
-	if (m->scan_cnt) {
-		for (i = 0; i < mod->len; i++)
-			free(m->scan_cnt[i]);
-		free(m->scan_cnt);
-	}
+	libxmp_free_scan(ctx);
 
 	free(m->comment);
+	m->comment = NULL;
 
 	D_("free dirname/basename");
 	free(m->dirname);
 	free(m->basename);
+	m->basename = NULL;
+	m->dirname = NULL;
 }
 
 void xmp_scan_module(xmp_context opaque)
