@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2016 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2021 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -53,20 +53,44 @@ static int dbm_test(HIO_HANDLE * f, char *t, const int start)
 
 
 struct local_data {
+	int have_info;
 	int have_song;
+	int have_patt;
+	int have_smpl;
+	int have_inst;
+	int have_venv;
+	int have_penv;
+	int maj_version;
+	int min_version;
+};
+
+struct dbm_envelope {
+	int ins;
+	int flg;
+	int npt;
+	int sus;
+	int lps;
+	int lpe;
+	int sus2;
+	struct dbm_envelope_node {
+		uint16 position;
+		int16 value;
+	} nodes[32];
 };
 
 
 static int get_info(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 {
 	struct xmp_module *mod = &m->mod;
+	struct local_data *data = (struct local_data *)parm;
 	int val;
 
 	/* Sanity check */
-	if (mod->ins != 0) {
+	if (data->have_info || size < 10) {
 		return -1;
 	}
- 
+	data->have_info = 1;
+
 	val = hio_read16b(f);
 	if (val < 0 || val > 255) {
 		D_(D_CRIT "Invalid number of instruments: %d", val);
@@ -121,13 +145,14 @@ static int get_song(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 	int i;
 	char buffer[50];
 
-	if (data->have_song)
+	/* Sanity check */
+	if (data->have_song || size < 46) {
 		return 0;
-
+	}
 	data->have_song = 1;
 
 	hio_read(buffer, 44, 1, f);
-	D_(D_INFO "Song name: %s", buffer);
+	D_(D_INFO "Song name: %.44s", buffer);
 
 	mod->len = hio_read16b(f);
 	D_(D_INFO "Song length: %d patterns", mod->len);
@@ -146,10 +171,17 @@ static int get_song(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 static int get_inst(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 {
 	struct xmp_module *mod = &m->mod;
+	struct local_data *data = (struct local_data *)parm;
 	int i;
 	int c2spd, flags, snum;
 	uint8 buffer[50];
 
+	/* Sanity check */
+	if (data->have_inst || size < 50 * mod->ins) {
+		return -1;
+	}
+	data->have_inst = 1;
+
 	D_(D_INFO "Instruments: %d", mod->ins);
 
 	for (i = 0; i < mod->ins; i++) {
@@ -160,14 +192,17 @@ static int get_inst(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 		hio_read(buffer, 30, 1, f);
 		libxmp_instrument_name(mod, i, buffer, 30);
 		snum = hio_read16b(f);
-		if (snum == 0 || snum > mod->smp)
+		if (snum == 0 || snum > mod->smp) {
+			/* Skip remaining data for this instrument. */
+			hio_seek(f, 18, SEEK_CUR);
 			continue;
+		}
 
 		mod->xxi[i].sub[0].sid = --snum;
 		mod->xxi[i].sub[0].vol = hio_read16b(f);
 		c2spd = hio_read32b(f);
 		mod->xxs[snum].lps = hio_read32b(f);
-		mod->xxs[snum].lpe = mod->xxs[i].lps + hio_read32b(f);
+		mod->xxs[snum].lpe = mod->xxs[snum].lps + hio_read32b(f);
 		mod->xxi[i].sub[0].pan = 0x80 + (int16)hio_read16b(f);
 		if (mod->xxi[i].sub[0].pan > 0xff)
 			mod->xxi[i].sub[0].pan = 0xff;
@@ -188,10 +223,17 @@ static int get_inst(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 static int get_patt(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 {
 	struct xmp_module *mod = &m->mod;
+	struct local_data *data = (struct local_data *)parm;
 	int i, c, r, n, sz;
 	struct xmp_event *event, dummy;
 	uint8 x;
 
+	/* Sanity check */
+	if (data->have_patt || !data->have_info) {
+		return -1;
+	}
+	data->have_patt = 1;
+
 	if (libxmp_init_pattern(mod) < 0)
 		return -1;
 
@@ -203,7 +245,11 @@ static int get_patt(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 	 */
 
 	for (i = 0; i < mod->pat; i++) {
-		if (libxmp_alloc_pattern_tracks(mod, i, hio_read16b(f)) < 0)
+		int rows = hio_read16b(f);
+		if (hio_error(f))
+			return -1;
+
+		if (libxmp_alloc_pattern_tracks(mod, i, rows) < 0)
 			return -1;
 
 		sz = hio_read32b(f);
@@ -215,6 +261,9 @@ static int get_patt(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 		while (sz > 0) {
 			//printf("  offset=%x,  sz = %d, ", hio_tell(f), sz);
 			c = hio_read8(f);
+			if (hio_error(f))
+				return -1;
+
 			if (--sz <= 0) break;
 			//printf("c = %02x\n", c);
 
@@ -282,8 +331,15 @@ static int get_patt(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 static int get_smpl(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 {
 	struct xmp_module *mod = &m->mod;
+	struct local_data *data = (struct local_data *)parm;
 	int i, flags;
 
+	/* Sanity check */
+	if (data->have_smpl || !data->have_info) {
+		return -1;
+	}
+	data->have_smpl = 1;
+
 	D_(D_INFO "Stored samples: %d", mod->smp);
 
 	for (i = 0; i < mod->smp; i++) {
@@ -299,7 +355,7 @@ static int get_smpl(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 			hio_seek(f, mod->xxs[i].len, SEEK_CUR);
 			continue;
 		}
-		
+
 		if (libxmp_load_sample(m, f, SAMPLE_FLAG_BIGEND, &mod->xxs[i], NULL) < 0)
 			return -1;
 
@@ -318,28 +374,114 @@ static int get_smpl(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 	return 0;
 }
 
+static int read_envelope(struct xmp_module *mod, struct dbm_envelope *env, HIO_HANDLE *f)
+{
+	int i;
+
+	env->ins  = (int)hio_read16b(f) - 1;
+	env->flg  = hio_read8(f) & 0x7;
+	env->npt  = (int)hio_read8(f) + 1; /* DBM counts sections, not points. */
+	env->sus  = hio_read8(f);
+	env->lps  = hio_read8(f);
+	env->lpe  = hio_read8(f);
+	env->sus2 = hio_read8(f);
+
+	/* The format document claims there should be a reserved byte here but
+	 * no DigiBooster Pro module actually has this. The revised document
+	 * on the DigiBooster 3 website is corrected.
+	 */
+
+	/* Sanity check */
+	if (env->ins < 0 || env->ins >= mod->ins || env->npt > 32 ||
+	    env->sus >= 32 || env->lps >= 32 || env->lpe >= 32)
+		return -1;
+
+	for (i = 0; i < 32; i++) {
+		env->nodes[i].position	= hio_read16b(f);
+		env->nodes[i].value	= (int16)hio_read16b(f);
+	}
+
+	if (hio_error(f))
+		return -1;
+
+	return 0;
+}
+
 static int get_venv(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 {
 	struct xmp_module *mod = &m->mod;
+	struct local_data *data = (struct local_data *)parm;
+	struct dbm_envelope env;
 	int i, j, nenv, ins;
 
+	/* Sanity check */
+	if (data->have_venv || !data->have_info) {
+		return -1;
+	}
+	data->have_venv = 1;
+
 	nenv = hio_read16b(f);
 
 	D_(D_INFO "Vol envelopes  : %d ", nenv);
 
 	for (i = 0; i < nenv; i++) {
-		ins = hio_read16b(f) - 1;
-		mod->xxi[ins].aei.flg = hio_read8(f) & 0x07;
-		mod->xxi[ins].aei.npt = hio_read8(f);
-		mod->xxi[ins].aei.sus = hio_read8(f);
-		mod->xxi[ins].aei.lps = hio_read8(f);
-		mod->xxi[ins].aei.lpe = hio_read8(f);
-		hio_read8(f);	/* 2nd sustain */
-		//hio_read8(f);	/* reserved */
+		if (read_envelope(mod, &env, f) != 0)
+			return -1;
+
+		ins = env.ins;
+		mod->xxi[ins].aei.flg = env.flg;
+		mod->xxi[ins].aei.npt = env.npt;
+		mod->xxi[ins].aei.sus = env.sus;
+		mod->xxi[ins].aei.lps = env.lps;
+		mod->xxi[ins].aei.lpe = env.lpe;
 
 		for (j = 0; j < 32; j++) {
-			mod->xxi[ins].aei.data[j * 2 + 0] = hio_read16b(f);
-			mod->xxi[ins].aei.data[j * 2 + 1] = hio_read16b(f);
+			mod->xxi[ins].aei.data[j * 2 + 0] = env.nodes[j].position;
+			mod->xxi[ins].aei.data[j * 2 + 1] = env.nodes[j].value;
+		}
+	}
+
+	return 0;
+}
+
+static int get_penv(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
+{
+	struct xmp_module *mod = &m->mod;
+	struct local_data *data = (struct local_data *)parm;
+	struct dbm_envelope env;
+	int i, j, nenv, ins;
+
+	/* Sanity check */
+	if (data->have_penv || !data->have_info) {
+		return -1;
+	}
+	data->have_penv = 1;
+
+	nenv = hio_read16b(f);
+
+	D_(D_INFO "Pan envelopes  : %d ", nenv);
+
+	for (i = 0; i < nenv; i++) {
+		if (read_envelope(mod, &env, f) != 0)
+			return -1;
+
+		ins = env.ins;
+		mod->xxi[ins].pei.flg = env.flg;
+		mod->xxi[ins].pei.npt = env.npt;
+		mod->xxi[ins].pei.sus = env.sus;
+		mod->xxi[ins].pei.lps = env.lps;
+		mod->xxi[ins].pei.lpe = env.lpe;
+
+		for (j = 0; j < 32; j++) {
+			/* DigiBooster Pro 2 stores the pan value between 0 and 64.
+			 * DigiBooster 3 stores it from -128 to 128 (Krashan - M2.dbm).
+			 */
+			if (data->maj_version >= 3) {
+				env.nodes[j].value = env.nodes[j].value / 4 + 32;
+			}
+
+			mod->xxi[ins].pei.data[j * 2 + 0] = env.nodes[j].position;
+			mod->xxi[ins].pei.data[j * 2 + 1] = env.nodes[j].value;
 		}
 	}
 
@@ -350,7 +492,7 @@ static int dbm_load(struct module_data *m, HIO_HANDLE *f, const int start)
 {
 	struct xmp_module *mod = &m->mod;
 	iff_handle handle;
-	char name[44];
+	char name[XMP_NAME_SIZE];
 	uint16 version;
 	int i, ret;
 	struct local_data data;
@@ -359,17 +501,21 @@ static int dbm_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
 	hio_read32b(f);		/* DBM0 */
 
-	data.have_song = 0;
+	memset(&data, 0, sizeof(struct local_data));
 	version = hio_read16b(f);
+	data.maj_version = version >> 8;
+	data.min_version = version & 0xFF;
 
 	hio_seek(f, 10, SEEK_CUR);
 	hio_read(name, 1, 44, f);
+	name[44] = '\0';
 
 	handle = libxmp_iff_new();
 	if (handle == NULL)
 		return -1;
 
 	m->c4rate = C4_NTSC_RATE;
+	m->quirk |= QUIRK_FINEFX;
 
 	/* IFF chunk IDs */
 	ret = libxmp_iff_register(handle, "INFO", get_info);
@@ -378,13 +524,14 @@ static int dbm_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	ret |= libxmp_iff_register(handle, "PATT", get_patt);
 	ret |= libxmp_iff_register(handle, "SMPL", get_smpl);
 	ret |= libxmp_iff_register(handle, "VENV", get_venv);
+	ret |= libxmp_iff_register(handle, "PENV", get_penv);
 
 	if (ret != 0)
 		return -1;
 
 	strncpy(mod->name, name, XMP_NAME_SIZE);
 	snprintf(mod->type, XMP_NAME_SIZE, "DigiBooster Pro %d.%02x DBM0",
-					version >> 8, version & 0xff);
+					data.maj_version, data.min_version);
 
 	MODULE_INFO();
 
