@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2016 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2021 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -20,8 +20,6 @@
  * THE SOFTWARE.
  */
 
-#include <assert.h>
-
 #include "loader.h"
 #include "iff.h"
 #include "period.h"
@@ -121,10 +119,20 @@ static int get_patt(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 	struct xmp_module *mod = &m->mod;
 	struct local_data *data = (struct local_data *)parm;
 
+	/* Sanity check */
+	if (data->pflag) {
+		return -1;
+	}
+
 	mod->chn = hio_read16b(f);
 	data->realpat = hio_read16b(f);
 	mod->trk = mod->chn * mod->pat;
 
+	/* Sanity check */
+	if (mod->chn > XMP_MAX_CHANNELS) {
+		return -1;
+	}
+
 	return 0;
 }
 
@@ -134,8 +142,18 @@ static int get_inst(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 	int i, c2spd;
 	uint8 name[30];
 
+	/* Sanity check */
+	if (mod->ins != 0) {
+		return -1;
+	}
+
 	mod->ins = mod->smp = hio_read16b(f);
 
+	/* Sanity check */
+	if (mod->ins > MAX_INSTRUMENTS) {
+		return -1;
+	}
+
 	D_(D_INFO "Instruments    : %d ", mod->ins);
 
 	if (libxmp_init_instrument(m) < 0)
@@ -294,7 +312,7 @@ static int dt_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	LOAD_INIT();
 
 	memset(&data, 0, sizeof (struct local_data));
-	
+
 	handle = libxmp_iff_new();
 	if (handle == NULL)
 		return -1;
