@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2016 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2021 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -22,7 +22,6 @@
 
 #ifndef LIBXMP_CORE_DISABLE_IT
 
-#include <time.h>
 #include "loader.h"
 #include "it.h"
 #include "period.h"
@@ -41,16 +40,7 @@ const struct format_loader libxmp_loader_it = {
 	it_load
 };
 
-#ifdef WIN32
-struct tm *localtime_r(const time_t * timep, struct tm *result)
-{
-	/* Note: Win32 localtime() is thread-safe */
-	memcpy(result, localtime(timep), sizeof(struct tm));
-	return result;
-}
-#endif
-
-static int it_test(HIO_HANDLE * f, char *t, const int start)
+static int it_test(HIO_HANDLE *f, char *t, const int start)
 {
 	if (hio_read32b(f) != MAGIC_IMPM)
 		return -1;
@@ -66,7 +56,7 @@ static int it_test(HIO_HANDLE * f, char *t, const int start)
 #define L_CHANNELS 64
 
 
-static const uint8 fx[] = {
+static const uint8 fx[32] = {
 	/*   */ FX_NONE,
 	/* A */ FX_S3M_SPEED,
 	/* B */ FX_JUMP,
@@ -93,7 +83,12 @@ static const uint8 fx[] = {
 	/* W */ FX_GVOL_SLIDE,
 	/* X */ FX_SETPAN,
 	/* Y */ FX_PANBRELLO,
-	/* Z */ FX_FLT_CUTOFF
+	/* Z */ FX_FLT_CUTOFF,
+	/* ? */ FX_NONE,
+	/* ? */ FX_NONE,
+	/* ? */ FX_NONE,
+	/* ? */ FX_NONE,
+	/* ? */ FX_NONE
 };
 
 
@@ -263,33 +258,29 @@ static void fix_name(uint8 *s, int l)
 }
 
 
-static void read_envelope(struct xmp_envelope *ei, struct it_envelope *env,
+static int read_envelope(struct xmp_envelope *ei, struct it_envelope *env,
 			  HIO_HANDLE *f)
 {
-	int j;
+	int i;
+	uint8 buf[82];
 
-	env->flg = hio_read8(f);
-	env->num = hio_read8(f);
-
-	/* Sanity check */
-	if (env->num >= XMP_MAX_ENV_POINTS) {
-		env->flg = 0;
-		env->num = 0;
-		return;
+	if (hio_read(buf, 1, 82, f) != 82) {
+		return -1;
 	}
 
-	env->lpb = hio_read8(f);
-	env->lpe = hio_read8(f);
-	env->slb = hio_read8(f);
-	env->sle = hio_read8(f);
+	env->flg = buf[0];
+	env->num = MIN(buf[1], 25); /* Clamp to IT max */
 
-	for (j = 0; j < 25; j++) {
-		env->node[j].y = hio_read8(f);
-		env->node[j].x = hio_read16l(f);
+	env->lpb = buf[2];
+	env->lpe = buf[3];
+	env->slb = buf[4];
+	env->sle = buf[5];
+
+	for (i = 0; i < 25; i++) {
+		env->node[i].y = buf[6 + i * 3];
+		env->node[i].x = readmem16l(buf + 7 + i * 3);
 	}
 
-	env->unused = hio_read8(f);
-
 	ei->flg = env->flg & IT_ENV_ON ? XMP_ENVELOPE_ON : 0;
 
 	if (env->flg & IT_ENV_LOOP) {
@@ -310,14 +301,16 @@ static void read_envelope(struct xmp_envelope *ei, struct it_envelope *env,
 	ei->lps = env->lpb;
 	ei->lpe = env->lpe;
 
-	if (ei->npt > 0 && ei->npt <= 25 /* XMP_MAX_ENV_POINTS */ ) {
-		for (j = 0; j < ei->npt; j++) {
-			ei->data[j * 2] = env->node[j].x;
-			ei->data[j * 2 + 1] = env->node[j].y;
+	if (ei->npt > 0 && ei->npt <= 25 /* XMP_MAX_ENV_POINTS */) {
+		for (i = 0; i < ei->npt; i++) {
+			ei->data[i * 2] = env->node[i].x;
+			ei->data[i * 2 + 1] = env->node[i].y;
 		}
 	} else {
 		ei->flg &= ~XMP_ENVELOPE_ON;
 	}
+
+	return 0;
 }
 
 static void identify_tracker(struct module_data *m, struct it_file_header *ifh)
@@ -367,25 +360,10 @@ static void identify_tracker(struct module_data *m, struct it_file_header *ifh)
 		break;
 	default:
 		switch (ifh->cwt >> 12) {
-		case 0x1:{
-			uint16 cwtv = ifh->cwt & 0x0fff;
-			struct tm version;
-			time_t version_sec;
-
-			if (cwtv > 0x50) {
-				version_sec = ((cwtv - 0x050) * 86400) + 1254355200;
-				if (localtime_r(&version_sec, &version)) {
-					snprintf(tracker_name, 40,
-						 "Schism Tracker %04d-%02d-%02d",
-						 version.tm_year + 1900,
-						 version.tm_mon + 1,
-						 version.tm_mday);
-				}
-			} else {
-				snprintf(tracker_name, 40,
-					 "Schism Tracker 0.%x", cwtv);
-			}
-			break; }
+		case 0x1:
+			libxmp_schism_tracker_string(tracker_name, 40,
+				(ifh->cwt & 0x0fff), ifh->rsvd);
+			break;
 		case 0x5:
 			snprintf(tracker_name, 40, "OpenMPT %d.%02x",
 				 (ifh->cwt & 0x0f00) >> 8, ifh->cwt & 0xff);
@@ -413,43 +391,42 @@ static int load_old_it_instrument(struct xmp_instrument *xxi, HIO_HANDLE *f)
 	int inst_map[120], inst_rmap[XMP_MAX_KEYS];
 	struct it_instrument1_header i1h;
 	int c, k, j;
+	uint8 buf[64];
 
-	i1h.magic = hio_read32b(f);
-	hio_read(&i1h.dosname, 12, 1, f);
-
-	i1h.zero = hio_read8(f);
-	i1h.flags = hio_read8(f);
-	i1h.vls = hio_read8(f);
-	i1h.vle = hio_read8(f);
-	i1h.sls = hio_read8(f);
-	i1h.sle = hio_read8(f);
-	i1h.rsvd1 = hio_read16l(f);
-	i1h.fadeout = hio_read16l(f);
-
-	i1h.nna = hio_read8(f);
-	i1h.dnc = hio_read8(f);
-	i1h.trkvers = hio_read16l(f);
-	i1h.nos = hio_read8(f);
-	i1h.rsvd2 = hio_read8(f);
-
-	if (hio_error(f)) {
+	if (hio_read(buf, 1, 64, f) != 64) {
 		return -1;
 	}
 
-	if (hio_read(&i1h.name, 1, 26, f) != 26) {
+	i1h.magic = readmem32b(buf);
+	if (i1h.magic != MAGIC_IMPI) {
+		D_(D_CRIT "bad instrument magic");
 		return -1;
 	}
+	memcpy(i1h.dosname, buf + 4, 12);
+	i1h.zero = buf[16];
+	i1h.flags = buf[17];
+	i1h.vls = buf[18];
+	i1h.vle = buf[19];
+	i1h.sls = buf[20];
+	i1h.sle = buf[21];
+	i1h.fadeout = readmem16l(buf + 24);
+	i1h.nna = buf[26];
+	i1h.dnc = buf[27];
+	i1h.trkvers = readmem16l(buf + 28);
+	i1h.nos = buf[30];
 
+	memcpy(i1h.name, buf + 32, 26);
 	fix_name(i1h.name, 26);
 
-	if (hio_read(&i1h.rsvd3, 1, 6, f) != 6)
+	if (hio_read(i1h.keys, 1, 240, f) != 240) {
 		return -1;
-	if (hio_read(&i1h.keys, 1, 240, f) != 240)
+	}
+	if (hio_read(i1h.epoint, 1, 200, f) != 200) {
 		return -1;
-	if (hio_read(&i1h.epoint, 1, 200, f) != 200)
-		return -1;
-	if (hio_read(&i1h.enode, 1, 50, f) != 50)
+	}
+	if (hio_read(i1h.enode, 1, 50, f) != 50) {
 		return -1;
+	}
 
 	libxmp_copy_adjust(xxi->name, i1h.name, 25);
 
@@ -544,19 +521,24 @@ static int load_new_it_instrument(struct xmp_instrument *xxi, HIO_HANDLE *f)
 	int inst_map[120], inst_rmap[XMP_MAX_KEYS];
 	struct it_instrument2_header i2h;
 	struct it_envelope env;
-	int dca2nna[] = { 0, 2, 3 };
+	int dca2nna[] = { 0, 2, 3, 3 /* Northern Sky (cj-north.it) has this... */ };
 	int c, k, j;
+	uint8 buf[64];
 
-	i2h.magic = hio_read32b(f);
+	if (hio_read(buf, 1, 64, f) != 64) {
+		return -1;
+	}
+
+	i2h.magic = readmem32b(buf);
 	if (i2h.magic != MAGIC_IMPI) {
 		D_(D_CRIT "bad instrument magic");
 		return -1;
 	}
-	hio_read(&i2h.dosname, 12, 1, f);
-	i2h.zero = hio_read8(f);
-	i2h.nna = hio_read8(f);
-	i2h.dct = hio_read8(f);
-	i2h.dca = hio_read8(f);
+	memcpy(i2h.dosname, buf + 4, 12);
+	i2h.zero = buf[16];
+	i2h.nna = buf[17];
+	i2h.dct = buf[18];
+	i2h.dca = buf[19];
 
 	/* Sanity check */
 	if (i2h.dca > 3) {
@@ -565,38 +547,26 @@ static int load_new_it_instrument(struct xmp_instrument *xxi, HIO_HANDLE *f)
 		i2h.dca = 0;
 	}
 
-	i2h.fadeout = hio_read16l(f);
-
-	i2h.pps = hio_read8(f);
-	i2h.ppc = hio_read8(f);
-	i2h.gbv = hio_read8(f);
-	i2h.dfp = hio_read8(f);
-	i2h.rv = hio_read8(f);
-	i2h.rp = hio_read8(f);
-	i2h.trkvers = hio_read16l(f);
-
-	i2h.nos = hio_read8(f);
-	i2h.rsvd1 = hio_read8(f);
-
-	if (hio_error(f)) {
-		D_(D_CRIT "read error");
-		return -1;
-	}
-
-	if (hio_read(&i2h.name, 1, 26, f) != 26) {
-		D_(D_CRIT "name read error");
-		return -1;
-	}
+	i2h.fadeout = readmem16l(buf + 20);
+	i2h.pps = buf[22];
+	i2h.ppc = buf[23];
+	i2h.gbv = buf[24];
+	i2h.dfp = buf[25];
+	i2h.rv = buf[26];
+	i2h.rp = buf[27];
+	i2h.trkvers = readmem16l(buf + 28);
+	i2h.nos = buf[30];
 
+	memcpy(i2h.name, buf + 32, 26);
 	fix_name(i2h.name, 26);
 
-	i2h.ifc = hio_read8(f);
-	i2h.ifr = hio_read8(f);
-	i2h.mch = hio_read8(f);
-	i2h.mpr = hio_read8(f);
-	i2h.mbnk = hio_read16l(f);
+	i2h.ifc = buf[58];
+	i2h.ifr = buf[59];
+	i2h.mch = buf[60];
+	i2h.mpr = buf[61];
+	i2h.mbnk = readmem16l(buf + 62);
 
-	if (hio_read(&i2h.keys, 1, 240, f) != 240) {
+	if (hio_read(i2h.keys, 1, 240, f) != 240) {
 		D_(D_CRIT "key map read error");
 		return -1;
 	}
@@ -606,21 +576,30 @@ static int load_new_it_instrument(struct xmp_instrument *xxi, HIO_HANDLE *f)
 
 	/* Envelopes */
 
-	read_envelope(&xxi->aei, &env, f);
-	read_envelope(&xxi->pei, &env, f);
-	read_envelope(&xxi->fei, &env, f);
+	if (read_envelope(&xxi->aei, &env, f) < 0) {
+		return -1;
+	}
+	if (read_envelope(&xxi->pei, &env, f) < 0) {
+		return -1;
+	}
+	if (read_envelope(&xxi->fei, &env, f) < 0) {
+		return -1;
+	}
 
 	if (xxi->pei.flg & XMP_ENVELOPE_ON) {
 		for (j = 0; j < xxi->pei.npt; j++)
 			xxi->pei.data[j * 2 + 1] += 32;
 	}
 
-	if (xxi->aei.flg & XMP_ENVELOPE_ON && xxi->aei.npt == 0)
+	if (xxi->aei.flg & XMP_ENVELOPE_ON && xxi->aei.npt == 0) {
 		xxi->aei.npt = 1;
-	if (xxi->pei.flg & XMP_ENVELOPE_ON && xxi->pei.npt == 0)
+	}
+	if (xxi->pei.flg & XMP_ENVELOPE_ON && xxi->pei.npt == 0) {
 		xxi->pei.npt = 1;
-	if (xxi->fei.flg & XMP_ENVELOPE_ON && xxi->fei.npt == 0)
+	}
+	if (xxi->fei.flg & XMP_ENVELOPE_ON && xxi->fei.npt == 0) {
 		xxi->fei.npt = 1;
+	}
 
 	if (env.flg & IT_ENV_FILTER) {
 		xxi->fei.flg |= XMP_ENVELOPE_FLT;
@@ -692,6 +671,17 @@ static int load_new_it_instrument(struct xmp_instrument *xxi, HIO_HANDLE *f)
 	return 0;
 }
 
+static void force_sample_length(struct xmp_sample *xxs, int len)
+{
+	xxs->len = len;
+
+	if (xxs->lpe > xxs->len)
+		xxs->lpe = xxs->len;
+
+	if (xxs->lps >= xxs->len)
+		xxs->flg &= ~XMP_SAMPLE_LOOP;
+}
+
 static int load_it_sample(struct module_data *m, int i, int start,
 			  int sample_mode, HIO_HANDLE *f)
 {
@@ -699,6 +689,7 @@ static int load_it_sample(struct module_data *m, int i, int start,
 	struct xmp_module *mod = &m->mod;
 	struct xmp_sample *xxs, *xsmp;
 	int j, k;
+	uint8 buf[80];
 
 	if (sample_mode) {
 		mod->xxi[i].sub = calloc(sizeof(struct xmp_subinstrument), 1);
@@ -707,61 +698,49 @@ static int load_it_sample(struct module_data *m, int i, int start,
 		}
 	}
 
-	ish.magic = hio_read32b(f);
-	if (ish.magic != MAGIC_IMPS) {
+	if (hio_read(buf, 1, 80, f) != 80) {
 		return -1;
 	}
 
+	ish.magic = readmem32b(buf);
+	/* Changed to continue to allow use-brdg.it and use-funk.it to
+	 * load correctly (both IT 2.04)
+	 */
+	if (ish.magic != MAGIC_IMPS) {
+		return 0;
+	}
+
 	xxs = &mod->xxs[i];
 	xsmp = &m->xsmp[i];
 
-	hio_read(&ish.dosname, 12, 1, f);
-	ish.zero = hio_read8(f);
-	ish.gvl = hio_read8(f);
-	ish.flags = hio_read8(f);
-	ish.vol = hio_read8(f);
-
-	if (hio_error(f)) {
-		return -1;
-	}
-
-	if (hio_read(&ish.name, 1, 26, f) != 26) {
-		return -1;
-	}
+	memcpy(ish.dosname, buf + 4, 12);
+	ish.zero = buf[16];
+	ish.gvl = buf[17];
+	ish.flags = buf[18];
+	ish.vol = buf[19];
 
+	memcpy(ish.name, buf + 20, 26);
 	fix_name(ish.name, 26);
 
-	ish.convert = hio_read8(f);
-	ish.dfp = hio_read8(f);
-	ish.length = hio_read32l(f);
-	ish.loopbeg = hio_read32l(f);
-	ish.loopend = hio_read32l(f);
-	ish.c5spd = hio_read32l(f);
-	ish.sloopbeg = hio_read32l(f);
-	ish.sloopend = hio_read32l(f);
-	ish.sample_ptr = hio_read32l(f);
-
-	ish.vis = hio_read8(f);
-	ish.vid = hio_read8(f);
-	ish.vir = hio_read8(f);
-	ish.vit = hio_read8(f);
-
-	/* Changed to continue to allow use-brdg.it and use-funk.it to
-	 * load correctly (both IT 2.04)
-	 */
-	if (ish.magic != MAGIC_IMPS)
-		return 0;
+	ish.convert = buf[46];
+	ish.dfp = buf[47];
+	ish.length = readmem32l(buf + 48);
+	ish.loopbeg = readmem32l(buf + 52);
+	ish.loopend = readmem32l(buf + 56);
+	ish.c5spd = readmem32l(buf + 60);
+	ish.sloopbeg = readmem32l(buf + 64);
+	ish.sloopend = readmem32l(buf + 68);
+	ish.sample_ptr = readmem32l(buf + 72);
+	ish.vis = buf[76];
+	ish.vid = buf[77];
+	ish.vir = buf[78];
+	ish.vit = buf[79];
 
 	if (ish.flags & IT_SMP_16BIT) {
 		xxs->flg = XMP_SAMPLE_16BIT;
 	}
 	xxs->len = ish.length;
 
-	/* Sanity check */
-	if (xxs->len > MAX_SAMPLE_SIZE) {
-		return -1;
-	}
-
 	xxs->lps = ish.loopbeg;
 	xxs->lpe = ish.loopend;
 	xxs->flg |= ish.flags & IT_SMP_LOOP ? XMP_SAMPLE_LOOP : 0;
@@ -782,6 +761,7 @@ static int load_it_sample(struct module_data *m, int i, int start,
 
 	if (sample_mode) {
 		/* Create an instrument for each sample */
+		mod->xxi[i].vol = 64;
 		mod->xxi[i].sub[0].vol = ish.vol;
 		mod->xxi[i].sub[0].pan = 0x80;
 		mod->xxi[i].sub[0].sid = i;
@@ -793,7 +773,7 @@ static int load_it_sample(struct module_data *m, int i, int start,
 
 	D_(D_INFO "\n[%2X] %-26.26s %05x%c%05x %05x %05x %05x "
 	   "%02x%02x %02x%02x %5d ",
-	   i, sample_mode ? xxs->name : mod->xxi[i].name,
+	   i, sample_mode ? xxs->name : mod->xxs[i].name,
 	   xxs->len,
 	   ish.flags & IT_SMP_16BIT ? '+' : ' ',
 	   MIN(xxs->lps, 0xfffff), MIN(xxs->lpe, 0xfffff),
@@ -814,7 +794,7 @@ static int load_it_sample(struct module_data *m, int i, int start,
 				sub->vol = ish.vol;
 				sub->gvl = ish.gvl;
 				sub->vra = ish.vis;	/* sample to sub-instrument vibrato */
-				sub->vde = ish.vid >> 1;
+				sub->vde = ish.vid << 1;
 				sub->vwf = ish.vit;
 				sub->vsw = (0xff - ish.vir) >> 1;
 
@@ -835,6 +815,12 @@ static int load_it_sample(struct module_data *m, int i, int start,
 	if (ish.flags & IT_SMP_SAMPLE && xxs->len > 1) {
 		int cvt = 0;
 
+		/* Sanity check - some modules may have invalid sizes on
+		 * unused samples so only check this if the sample flag is set. */
+		if (xxs->len > MAX_SAMPLE_SIZE) {
+			return -1;
+		}
+
 		if (0 != hio_seek(f, start + ish.sample_ptr, SEEK_SET))
 			return -1;
 
@@ -846,9 +832,32 @@ static int load_it_sample(struct module_data *m, int i, int start,
 
 		/* compressed samples */
 		if (ish.flags & IT_SMP_COMP) {
+			long min_size, file_len, left;
 			uint8 *buf;
 			int ret;
 
+			/* Sanity check - the lower bound on IT compressed
+			 * sample size (in bytes) is a little over 1/8th of the
+			 * number of SAMPLES in the sample.
+			 */
+			file_len = hio_size(f);
+			min_size = xxs->len >> 3;
+			left = file_len - (long)ish.sample_ptr;
+			/* No data to read at all? Just skip it... */
+			if (left <= 0)
+				return 0;
+
+			if ((file_len > 0) && (left < min_size)) {
+				D_(D_WARN "sample %X failed minimum size check "
+				   "(len=%d, needs >=%ld bytes, %ld available): "
+				   "resizing to %ld",
+				   i, xxs->len, min_size, left, left << 3);
+
+				force_sample_length(xxs, left << 3);
+				if (ish.flags & IT_SMP_SLOOP)
+					force_sample_length(xsmp, left << 3);
+			}
+
 			buf = calloc(1, xxs->len * 2);
 			if (buf == NULL)
 				return -1;
@@ -923,15 +932,16 @@ static int load_it_pattern(struct module_data *m, int i, int new_fx,
 
 	r = 0;
 
-	memset(last_fxp, 0, 64);
+	memset(last_fxp, 0, sizeof(last_fxp));
 	memset(lastevent, 0, L_CHANNELS * sizeof(struct xmp_event));
 	memset(&dummy, 0, sizeof(struct xmp_event));
 
 	pat_len = hio_read16l(f) /* - 4 */ ;
 	mod->xxp[i]->rows = hio_read16l(f);
 
-	if (libxmp_alloc_tracks_in_pattern(mod, i) < 0)
+	if (libxmp_alloc_tracks_in_pattern(mod, i) < 0) {
 		return -1;
+	}
 
 	memset(mask, 0, L_CHANNELS);
 	hio_read16l(f);
@@ -939,6 +949,9 @@ static int load_it_pattern(struct module_data *m, int i, int new_fx,
 
 	while (--pat_len >= 0) {
 		b = hio_read8(f);
+		if (hio_error(f)) {
+			return -1;
+		}
 		if (!b) {
 			r++;
 			continue;
@@ -1000,14 +1013,18 @@ static int load_it_pattern(struct module_data *m, int i, int new_fx,
 		}
 		if (mask[c] & 0x08) {
 			b = hio_read8(f);
-			if (b > 26) {
-				return -1;
+			if (b >= ARRAY_SIZE(fx)) {
+				D_(D_WARN "invalid effect %#02x", b);
+				hio_read8(f);
+
+			} else {
+				event->fxt = b;
+				event->fxp = hio_read8(f);
+
+				xlat_fx(c, event, last_fxp, new_fx);
+				lastevent[c].fxt = event->fxt;
+				lastevent[c].fxp = event->fxp;
 			}
-			event->fxt = b;
-			event->fxp = hio_read8(f);
-			xlat_fx(c, event, last_fxp, new_fx);
-			lastevent[c].fxt = event->fxt;
-			lastevent[c].fxp = event->fxp;
 			pat_len -= 2;
 		}
 		if (mask[c] & 0x10) {
@@ -1048,7 +1065,7 @@ static int it_load(struct module_data *m, HIO_HANDLE *f, const int start)
 		return -1;
 	}
 
-	hio_read(&ifh.name, 26, 1, f);
+	hio_read(ifh.name, 26, 1, f);
 	ifh.hilite_min = hio_read8(f);
 	ifh.hilite_maj = hio_read8(f);
 
@@ -1071,6 +1088,7 @@ static int it_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
 	/* Sanity check */
 	if (ifh.gv > 0x80 || ifh.mv > 0x80) {
+		D_(D_CRIT "invalid gv (%u) or mv (%u)", ifh.gv, ifh.mv);
 		goto err;
 	}
 
@@ -1078,10 +1096,12 @@ static int it_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	ifh.msgofs = hio_read32l(f);
 	ifh.rsvd = hio_read32l(f);
 
-	hio_read(&ifh.chpan, 64, 1, f);
-	hio_read(&ifh.chvol, 64, 1, f);
+	hio_read(ifh.chpan, 64, 1, f);
+	hio_read(ifh.chvol, 64, 1, f);
 
-	strncpy(mod->name, (char *)ifh.name, XMP_NAME_SIZE);
+	memcpy(mod->name, ifh.name, sizeof(ifh.name));
+	/* sizeof(ifh.name) == 26, sizeof(mod->name) == 64. */
+	mod->name[sizeof(ifh.name)] = '\0';
 	mod->len = ifh.ordnum;
 	mod->ins = ifh.insnum;
 	mod->smp = ifh.smpnum;
@@ -1089,6 +1109,8 @@ static int it_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
 	/* Sanity check */
 	if (mod->ins > 255 || mod->smp > 255 || mod->pat > 255) {
+		D_(D_CRIT "invalid ins (%u), smp (%u), or pat (%u)",
+		   mod->ins, mod->smp, mod->pat);
 		goto err;
 	}
 
@@ -1117,10 +1139,6 @@ static int it_load(struct module_data *m, HIO_HANDLE *f, const int start)
 		m->period_type = PERIOD_LINEAR;
 	}
 
-	if (!sample_mode && ifh.cmwt >= 0x200) {
-		m->quirk |= QUIRK_INSVOL;
-	}
-
 	for (i = 0; i < 64; i++) {
 		struct xmp_channel *xxc = &mod->xxc[i];
 
@@ -1129,7 +1147,6 @@ static int it_load(struct module_data *m, HIO_HANDLE *f, const int start)
 		}
 
 		if (ifh.chpan[i] & 0x80) {	/* Channel mute */
-			ifh.chvol[i] = 0;
 			xxc->flg |= XMP_CHANNEL_MUTE;
 		}
 
@@ -1143,6 +1160,7 @@ static int it_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
 		xxc->vol = ifh.chvol[i];
 	}
+
 	if (mod->len <= XMP_MAX_MOD_LENGTH) {
 		hio_read(mod->xxo, 1, mod->len, f);
 	} else {
@@ -1229,8 +1247,10 @@ static int it_load(struct module_data *m, HIO_HANDLE *f, const int start)
 			goto err4;
 		}
 	}
+	/* Reset any error status set by truncated samples. */
+	hio_error(f);
 
-	D_(D_INFO "Stored Patterns: %d", mod->pat);
+	D_(D_INFO "Stored patterns: %d", mod->pat);
 
 	/* Effects in muted channels are processed, so scan patterns first to
 	 * see the real number of channels
@@ -1253,6 +1273,10 @@ static int it_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
 		while (--pat_len >= 0) {
 			int b = hio_read8(f);
+			if (hio_error(f)) {
+				D_(D_CRIT "error scanning pattern %d", i);
+				goto err4;
+			}
 			if (b == 0)
 				continue;
 
@@ -1291,14 +1315,16 @@ static int it_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	mod->chn = max_ch + 1;
 	mod->trk = mod->pat * mod->chn;
 
-	if (libxmp_init_pattern(mod) < 0)
+	if (libxmp_init_pattern(mod) < 0) {
 		goto err4;
+	}
 
 	/* Read patterns */
 	for (i = 0; i < mod->pat; i++) {
 
-		if (libxmp_alloc_pattern(mod, i) < 0)
+		if (libxmp_alloc_pattern(mod, i) < 0) {
 			goto err4;
+		}
 
 		/* If the offset to a pattern is 0, the pattern is empty */
 		if (pp_pat[i] == 0) {
@@ -1313,10 +1339,12 @@ static int it_load(struct module_data *m, HIO_HANDLE *f, const int start)
 		}
 
 		if (hio_seek(f, start + pp_pat[i], SEEK_SET) < 0) {
+			D_(D_CRIT "error seeking to %d", start + pp_pat[i]);
 			goto err4;
 		}
 
 		if (load_it_pattern(m, i, new_fx, f) < 0) {
+			D_(D_CRIT "error loading pattern %d", i);
 			goto err4;
 		}
 	}
@@ -1328,12 +1356,12 @@ static int it_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	/* Song message */
 
 	if (ifh.special & IT_HAS_MSG) {
-		if ((m->comment = malloc(ifh.msglen + 1)) != NULL) {
+		if ((m->comment = malloc(ifh.msglen)) != NULL) {
 			hio_seek(f, start + ifh.msgofs, SEEK_SET);
 
 			D_(D_INFO "Message length : %d", ifh.msglen);
 
-			for (j = 0; j < ifh.msglen; j++) {
+			for (j = 0; j + 1 < ifh.msglen; j++) {
 				int b = hio_read8(f);
 				if (b == '\r') {
 					b = '\n';
@@ -1343,13 +1371,16 @@ static int it_load(struct module_data *m, HIO_HANDLE *f, const int start)
 				}
 				m->comment[j] = b;
 			}
-			m->comment[j] = 0;
+
+			if (ifh.msglen > 0) {
+				m->comment[j] = 0;
+			}
 		}
 	}
 
 	/* Format quirks */
 
-	m->quirk |= QUIRKS_IT | QUIRK_ARPMEM;
+	m->quirk |= QUIRKS_IT | QUIRK_ARPMEM | QUIRK_INSVOL;
 
 	if (ifh.flags & IT_LINK_GXX) {
 		m->quirk |= QUIRK_PRENV;
