@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2016 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2021 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -25,7 +25,6 @@
  * from ftp://ftp.funet.fi/pub/amiga/fish/301-400/ff349
  */
 
-#include <assert.h>
 #include "loader.h"
 
 #define MAGIC_MED3	MAGIC4('M','E','D',3)
@@ -103,6 +102,7 @@ static int unpack_block(struct module_data *m, uint16 bnum, uint8 *from, uint16
 	uint16 fromn = 0, lmsk;
 	uint8 *fromst = from + 16, bcnt, *tmpto;
 	uint8 *patbuf, *to;
+	uint32 nibs_left = convsz * 2;
 	int i, j, trkn = mod->chn;
 
 	/*from += 16;*/
@@ -118,10 +118,10 @@ static int unpack_block(struct module_data *m, uint16 bnum, uint8 *from, uint16
 		}
 
 		if (*lmptr & MASK) {
-			if (trkn / 2 > convsz) {
+			if (trkn / 4 > nibs_left) {
 				goto err2;
-			}	
-			convsz -= trkn / 2;
+			}
+			nibs_left -= trkn / 4;
 
 			lmsk = get_nibbles(fromst, &fromn, (uint8)(trkn / 4));
 			lmsk <<= (16 - trkn);
@@ -129,6 +129,10 @@ static int unpack_block(struct module_data *m, uint16 bnum, uint8 *from, uint16
 
 			for (bcnt = 0; bcnt < trkn; bcnt++) {
 				if (lmsk & 0x8000) {
+					if (nibs_left < 3) {
+						goto err2;
+					}
+					nibs_left -= 3;
 					*tmpto = (uint8)get_nibbles(fromst,
 						&fromn,2);
 					*(tmpto + 1) = (get_nibble(fromst,
@@ -140,10 +144,10 @@ static int unpack_block(struct module_data *m, uint16 bnum, uint8 *from, uint16
 		}
 
 		if (*fxptr & MASK) {
-			if (trkn / 2 > convsz) {
+			if (trkn / 4 > nibs_left) {
 				goto err2;
-			}	
-			convsz -= trkn / 2;
+			}
+			nibs_left -= trkn / 4;
 
 			lmsk = get_nibbles(fromst,&fromn,(uint8)(trkn / 4));
 			lmsk <<= (16 - trkn);
@@ -151,6 +155,10 @@ static int unpack_block(struct module_data *m, uint16 bnum, uint8 *from, uint16
 
 			for (bcnt = 0; bcnt < trkn; bcnt++) {
 				if (lmsk & 0x8000) {
+					if (nibs_left < 3) {
+						goto err2;
+					}
+					nibs_left -= 3;
 					*(tmpto+1) |= get_nibble(fromst,
 							&fromn);
 					*(tmpto+2) = (uint8)get_nibbles(fromst,
