@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2016 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2021 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -82,9 +82,9 @@ static int mtm_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
 	LOAD_INIT();
 
-	hio_read(&mfh.magic, 3, 1, f);	/* "MTM" */
+	hio_read(mfh.magic, 3, 1, f);	/* "MTM" */
 	mfh.version = hio_read8(f);	/* MSN=major, LSN=minor */
-	hio_read(&mfh.name, 20, 1, f);	/* ASCIIZ Module name */
+	hio_read(mfh.name, 20, 1, f);	/* ASCIIZ Module name */
 	mfh.tracks = hio_read16l(f);	/* Number of tracks saved */
 	mfh.patterns = hio_read8(f);	/* Number of patterns saved */
 	mfh.modlen = hio_read8(f);	/* Module length */
@@ -102,11 +102,11 @@ static int mtm_load(struct module_data *m, HIO_HANDLE *f, const int start)
 		return -1;
 
 	mfh.channels = hio_read8(f);	/* Number of tracks per pattern */
-	if (mfh.channels > XMP_MAX_CHANNELS) {
+	if (mfh.channels > MIN(32, XMP_MAX_CHANNELS)) {
 		return -1;
 	}
 
-	hio_read(&mfh.pan, 32, 1, f);	/* Pan positions for each channel */
+	hio_read(mfh.pan, 32, 1, f);	/* Pan positions for each channel */
 
 	if (hio_error(f)) {
 		return -1;
@@ -146,7 +146,7 @@ static int mtm_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
 		sub = &xxi->sub[0];
 
-		hio_read(&mih.name, 22, 1, f);	/* Instrument name */
+		hio_read(mih.name, 22, 1, f);	/* Instrument name */
 		mih.length = hio_read32l(f);	/* Instrument length in bytes */
 
 		if (mih.length > MAX_SAMPLE_SIZE)
@@ -200,14 +200,15 @@ static int mtm_load(struct module_data *m, HIO_HANDLE *f, const int start)
 		if (i == 0)
 			continue;
 
-		if (hio_read(&mt, 3, 64, f) != 64)
+		if (hio_read(mt, 3, 64, f) != 64)
 			return -1;
 
 		for (j = 0; j < 64; j++) {
 			struct xmp_event *e = &mod->xxt[i]->event[j];
 			uint8 *d = mt + j * 3;
 
-			if ((e->note = d[0] >> 2)) {
+			e->note = d[0] >> 2;
+			if (e->note) {
 				e->note += 37;
 			}
 			e->ins = ((d[0] & 0x3) << 4) + MSN(d[1]);
