@@ -6,8 +6,6 @@
  * Modified in 2006,2007,2014 by Claudio Matsuoka
  */
 
-#include <string.h>
-#include <stdlib.h>
 #include "prowiz.h"
 
 
@@ -17,7 +15,7 @@ static int write_event(uint8 c1, uint8 c2, uint8 fxp, FILE *out)
 	uint8 p[4];
 
 	note = ((c1 << 4) & 0x30) | ((c2 >> 4) & 0x0f);
-	if (note >= 37) {
+	if (!PTK_IS_VALID_NOTE(note)) {
 		/* di.nightmare has note 49! */
 		uint32 x = 0;
 		fwrite(&x, 4, 1, out);
@@ -48,8 +46,8 @@ static int depack_di(HIO_HANDLE *in, FILE *out)
 	int size, ssize;
 	int pos;
 
-	memset(ptable, 0, 128);
-	memset(paddr, 0, 256);
+	memset(ptable, 0, sizeof(ptable));
+	memset(paddr, 0, sizeof(paddr));
 
 	pw_write_zero(out, 20);			/* title */
 
@@ -74,7 +72,7 @@ static int depack_di(HIO_HANDLE *in, FILE *out)
 		write16b(out, hio_read16b(in));		/* loop size */
 	}
 
-	memset(tmp, 0, 50);
+	memset(tmp, 0, sizeof(tmp));
 	for (i = nins; i < 31; i++) {
 		fwrite(tmp, 30, 1, out);
 	}
@@ -101,6 +99,11 @@ static int depack_di(HIO_HANDLE *in, FILE *out)
 			max = ptable[i];
 	}
 
+	/* Sanity check */
+	if (max >= 128) {
+		return -1;
+	}
+
 	write32b(out, PW_MOD_MAGIC);
 
 	hio_seek(in, pos, SEEK_SET);
@@ -137,7 +140,7 @@ static int depack_di(HIO_HANDLE *in, FILE *out)
 }
 
 
-static int test_di(uint8 *data, char *t, int s)
+static int test_di(const uint8 *data, char *t, int s)
 {
 	int i;
 	int numsmp, ssize, psize;
@@ -164,7 +167,7 @@ static int test_di(uint8 *data, char *t, int s)
 		int len = readmem16b(data + 14) << 1;
 		int start = readmem16b(data + 18) << 1;
 		int lsize = readmem16b(data + 20) << 1;
-		uint8 *d = data + i * 8;
+		const uint8 *d = data + i * 8;
 
 		if (len > 0xffff || start > 0xffff || lsize > 0xffff)
 			return -1;
@@ -216,7 +219,7 @@ static int test_di(uint8 *data, char *t, int s)
 	}
 #endif
 
-	PW_REQUEST_DATA(s, pat_offs - 1);
+	PW_REQUEST_DATA(s, pat_offs);
 
 	/* test pattern table reliability */
 	for (i = ptab_offs; i < pat_offs - 1; i++) {
