@@ -6,8 +6,6 @@
  * Modified in 2006,2007,2014 by Claudio Matsuoka
  */
 
-#include <string.h>
-#include <stdlib.h>
 #include "prowiz.h"
 
 
@@ -47,7 +45,7 @@ static int depack_eu(HIO_HANDLE *in, FILE *out)
 
 	/* the track data now ... */
 	for (i = 0; i < npat; i++) {
-		memset(tmp, 0, 1024);
+		memset(tmp, 0, sizeof(tmp));
 		for (j = 0; j < 4; j++) {
 			hio_seek(in, trk_addr[i][j], SEEK_SET);
 			for (k = 0; k < 64; k++) {
@@ -87,7 +85,7 @@ static int depack_eu(HIO_HANDLE *in, FILE *out)
 	return 0;
 }
 
-static int test_eu(uint8 *data, char *t, int s)
+static int test_eu(const uint8 *data, char *t, int s)
 {
 	int i;
 	int len, max_pat, smp_offs;
@@ -102,7 +100,7 @@ static int test_eu(uint8 *data, char *t, int s)
 
 	/* test #3  finetunes & volumes */
 	for (i = 0; i < 31; i++) {
-		uint8 *d = data + i * 30;
+		const uint8 *d = data + i * 30;
 		int size = readmem16b(d + 42) << 1;
 		int start = readmem16b(d + 46) << 1;
 		int lsize = readmem16b(d + 48) << 1;
@@ -163,6 +161,8 @@ static int test_eu(uint8 *data, char *t, int s)
 			min_trk = trk;
 	}
 
+	PW_REQUEST_DATA(s, max_trk);
+
 	/* test track datas */
 	/* last track wont be tested ... */
 	for (i = min_trk; i < max_trk; i++) {
