@@ -6,8 +6,6 @@
  * Modified in 2006,2007,2014 by Claudio Matsuoka
  */
 
-#include <string.h>
-#include <stdlib.h>
 #include "prowiz.h"
 
 
@@ -20,7 +18,7 @@ static int depack_fcm(HIO_HANDLE *in, FILE *out)
 	int i;
 	int size, ssize = 0;
 
-	memset(ptable, 0, 128);
+	memset(ptable, 0, sizeof(ptable));
 
 	hio_read32b(in);				/* bypass "FC-M" ID */
 	hio_read16b(in);				/* version number? */
@@ -68,7 +66,7 @@ static int depack_fcm(HIO_HANDLE *in, FILE *out)
 	return 0;
 }
 
-static int test_fcm(uint8 *data, char *t, int s)
+static int test_fcm(const uint8 *data, char *t, int s)
 {
 	int j;
 
