@@ -14,8 +14,6 @@
  * Modified in 2006,2007,2014 by Claudio Matsuoka
  */
 
-#include <string.h>
-#include <stdlib.h>
 #include "prowiz.h"
 
 
@@ -33,9 +31,9 @@ static int depack_fuzz(HIO_HANDLE *in, FILE *out)
 	int lps, lsz;
 	int i, j, k, l;
 
-	memset(tidx, 0, 128 * 16);
-	memset(tidx_real, 0, 128 * 4);
-	memset(ord, 0, 128);
+	memset(tidx, 0, sizeof(tidx));
+	memset(tidx_real, 0, sizeof(tidx_real));
+	memset(ord, 0, sizeof(ord));
 
 	hio_read32b(in);			/* bypass ID */
 	hio_read16b(in);			/* bypass 2 unknown bytes */
@@ -140,8 +138,8 @@ static int depack_fuzz(HIO_HANDLE *in, FILE *out)
 	l = 2118 + len * 16;
 
 	for (i = 0; i < npat; i++) {
-		memset(data, 0, 1024);
-		memset(track, 0, 4 << 8);
+		memset(data, 0, sizeof(data));
+		memset(track, 0, sizeof(track));
 
 		hio_seek(in, l + (tidx_real[i][0] << 8), SEEK_SET);
 		hio_read(track[0], 256, 1, in);
@@ -173,10 +171,12 @@ static int depack_fuzz(HIO_HANDLE *in, FILE *out)
 	return 0;
 }
 
-static int test_fuzz(uint8 *data, char *t, int s)
+static int test_fuzz(const uint8 *data, char *t, int s)
 {
 	int i;
 
+	PW_REQUEST_DATA(s, 2115);
+
 	if (readmem32b(data) != MAGIC4('M','1','.','0'))
 		return -1;
 
