@@ -6,8 +6,6 @@
  * Modified in 2006,2007,2014 by Claudio Matsuoka
  */
 
-#include <string.h>
-#include <stdlib.h>
 #include "prowiz.h"
 
 
@@ -21,7 +19,7 @@ static int depack_GMC(HIO_HANDLE *in, FILE *out)
 	long ssize = 0;
 	long i = 0, j = 0;
 
-	memset(ptable, 0, 128);
+	memset(ptable, 0, sizeof(ptable));
 
 	pw_write_zero(out, 20);			/* title */
 
@@ -69,7 +67,7 @@ static int depack_GMC(HIO_HANDLE *in, FILE *out)
 	/* pattern data */
 	hio_seek(in, 444, SEEK_SET);
 	for (i = 0; i <= max; i++) {
-		memset(tmp, 0, 1024);
+		memset(tmp, 0, sizeof(tmp));
 		hio_read(tmp, 1024, 1, in);
 		for (j = 0; j < 256; j++) {
 			switch (tmp[(j * 4) + 2] & 0x0f) {
@@ -104,7 +102,7 @@ static int depack_GMC(HIO_HANDLE *in, FILE *out)
 	return 0;
 }
 
-static int test_GMC(uint8 *data, char *t, int s)
+static int test_GMC(const uint8 *data, char *t, int s)
 {
 	int i, j, k;
 	int ssize, numpat;
@@ -123,7 +121,7 @@ static int test_GMC(uint8 *data, char *t, int s)
 	ssize = 0;
 	for (i = 0; i < 15; i++) {
 		int len, lsize;
-		uint8 *d = data + 16 * i;
+		const uint8 *d = data + 16 * i;
 
 		/* volumes */
 		if (d[7] > 0x40)
@@ -168,7 +166,7 @@ static int test_GMC(uint8 *data, char *t, int s)
 	for (i = 0; i < numpat; i++) {
 		for (j = 0; j < 256; j++) {
 			int offset = 444 + i * 1024 + j * 4;
-			uint8 *d;
+			const uint8 *d;
 
 			PW_REQUEST_DATA(s, offset + 4);
 
