@@ -6,8 +6,6 @@
  * Modified in 2006,2007,2014 by Claudio Matsuoka
  */
 
-#include <string.h>
-#include <stdlib.h>
 #include "prowiz.h"
 
 
@@ -25,9 +23,9 @@ static int depack_ksm(HIO_HANDLE *in, FILE *out)
 	int ssize = 0;
 	int i, j, k;
 
-	memset(plist, 0, 128);
-	memset(trknum, 0, 128 * 4);
-	memset(real_tnum, 0, 128 * 4);
+	memset(plist, 0, sizeof(plist));
+	memset(trknum, 0, sizeof(trknum));
+	memset(real_tnum, 0, sizeof(real_tnum));
 
 	/* title */
 	hio_seek(in, 2, SEEK_SET);
@@ -141,8 +139,8 @@ static int depack_ksm(HIO_HANDLE *in, FILE *out)
 
 	/* pattern data */
 	for (i = 0; i < c5; i++) {
-		memset(tmp, 0, 1024);
-		memset(tdata, 0, 192 * 4);
+		memset(tmp, 0, sizeof(tmp));
+		memset(tdata, 0, sizeof(tdata));
 
 		for (k = 0; k < 4; k++) {
 			hio_seek(in, 1536 + 192 * real_tnum[i][k], SEEK_SET);
@@ -156,7 +154,7 @@ static int depack_ksm(HIO_HANDLE *in, FILE *out)
 				uint8 *t = &tdata[k][j * 3];
 
 				/* Sanity check */
-				if (t[0] >= 37) {
+				if (!PTK_IS_VALID_NOTE(t[0])) {
 					return -1;
 				}
 
@@ -177,7 +175,7 @@ static int depack_ksm(HIO_HANDLE *in, FILE *out)
 	return 0;
 }
 
-static int test_ksm (uint8 *data, char *t, int s)
+static int test_ksm (const uint8 *data, char *t, int s)
 {
 	int i, j;
 	int max_trk;
@@ -215,11 +213,11 @@ static int test_ksm (uint8 *data, char *t, int s)
 	if (max_trk == 0)
 		return -1;
 
-	PW_REQUEST_DATA(s, 1536 + max_trk * 192 + 63 * 3);
+	PW_REQUEST_DATA(s, 1536 + max_trk * 192 + 64 * 3);
 
 	/* real test on tracks data starts now */
 	for (i = 0; i <= max_trk; i++) {
-		uint8 *d = data + 1536 + i * 192;
+		const uint8 *d = data + 1536 + i * 192;
 		for (j = 0; j < 64; j++) {
 			if (d[j * 3] > 0x24)
 				return -1;
