@@ -8,16 +8,14 @@
  * Modified in 2006,2007,2014 by Claudio Matsuoka
  */
 
-#include <string.h>
-#include <stdlib.h>
 #include "prowiz.h"
 
 
 static int depack_p18a(HIO_HANDLE *in, FILE *out)
 {
 	short pat_max;
-	int tmp_ptr;
 	int refmax;
+	int refsize;
 	uint8 pnum[128];
 	int paddr[128];
 	short pptr[64][256];
@@ -31,12 +29,12 @@ static int depack_p18a(HIO_HANDLE *in, FILE *out)
 	uint8 fin[31];
 	uint8 oldins[4];
 
-	memset(pnum, 0, 128);
-	memset(pptr, 0, 64 << 8);
-	memset(pat, 0, 128 * 1024);
-	memset(fin, 0, 31);
-	memset(oldins, 0, 4);
-	memset(paddr, 0, 128 * 4);
+	memset(pnum, 0, sizeof(pnum));
+	memset(pptr, 0, sizeof(pptr));
+	memset(pat, 0, sizeof(pat));
+	memset(fin, 0, sizeof(fin));
+	memset(oldins, 0, sizeof(oldins));
+	memset(paddr, 0, sizeof(paddr));
 
 	pw_write_zero(out, 20);				/* title */
 
@@ -69,11 +67,10 @@ static int depack_p18a(HIO_HANDLE *in, FILE *out)
 
 	/* ordering of patterns addresses */
 
-	tmp_ptr = 0;
+	pat_max = 0;
 	for (i = 0; i < num_pat; i++) {
 		if (i == 0) {
 			pnum[0] = 0;
-			tmp_ptr++;
 			continue;
 		}
 		for (j = 0; j < i; j++) {
@@ -83,11 +80,9 @@ static int depack_p18a(HIO_HANDLE *in, FILE *out)
 			}
 		}
 		if (j == i)
-			pnum[i] = tmp_ptr++;
+			pnum[i] = (++pat_max);
 	}
 
-	pat_max = tmp_ptr - 1;
-
 	fwrite(pnum, 128, 1, out);		/* pattern table */
 	write32b(out, PW_MOD_MAGIC);		/* M.K. */
 
@@ -112,12 +107,15 @@ static int depack_p18a(HIO_HANDLE *in, FILE *out)
 
 	/* read "reference table" */
 	refmax += 1;			/* 1st value is 0 ! */
-	i = refmax * 4;			/* each block is 4 bytes long */
-	if ((reftab = (uint8 *)malloc(i)) == NULL) {
+	refsize = refmax * 4;		/* each block is 4 bytes long */
+	if ((reftab = (uint8 *)malloc(refsize)) == NULL) {
 		return -1;
 	}
-	
-	hio_read(reftab, i, 1, in);
+
+	if (hio_read(reftab, refsize, 1, in) < 1) {
+		goto err;
+	}
+
 	hio_seek(in, 5226, SEEK_SET);	/* back to pattern data start */
 
 	for (j = 0; j <= pat_max; j++) {
@@ -129,6 +127,11 @@ static int depack_p18a(HIO_HANDLE *in, FILE *out)
 				int x = hio_read16b(in) << 2;
 				int fine, ins, per, fxt;
 
+				/* Sanity check */
+				if (x >= refsize || hio_error(in)) {
+					goto err;
+				}
+
 				memcpy(p, &reftab[x], 4);
 
 				ins = ((p[2] >> 4) & 0x0f) | (p[0] & 0xf0);
@@ -192,7 +195,7 @@ static int depack_p18a(HIO_HANDLE *in, FILE *out)
 	return -1;
 }
 
-static int test_p18a(uint8 * data, char *t, int s)
+static int test_p18a(const uint8 * data, char *t, int s)
 {
 	uint8 magic[] = {
 		0x60, 0x38, 0x60, 0x00, 0x00, 0xa0, 0x60, 0x00,
