@@ -6,8 +6,6 @@
  * Modified in 2016 by Claudio Matsuoka
  */
 
-#include <string.h>
-#include <stdlib.h>
 #include "prowiz.h"
 
 static int depack_pp10(HIO_HANDLE *in, FILE *out)
@@ -20,7 +18,7 @@ static int depack_pp10(HIO_HANDLE *in, FILE *out)
 	int i, j, k;
 	int ntrk, size, ssize = 0;
 
-	memset(trk_num, 0, 128 * 4);
+	memset(trk_num, 0, sizeof(trk_num));
 
 	pw_write_zero(out, 20);				/* write title */
 
@@ -70,7 +68,7 @@ static int depack_pp10(HIO_HANDLE *in, FILE *out)
 
 	/* track/pattern data */
 	for (i = 0; i < len; i++) {
-		memset(pdata, 0, 1024);
+		memset(pdata, 0, sizeof(pdata));
 		for (j = 0; j < 4; j++) {
 			hio_seek(in, 762 + (trk_num[j][i] << 8), SEEK_SET);
 			for (k = 0; k < 64; k++) {
@@ -91,7 +89,7 @@ static int depack_pp10(HIO_HANDLE *in, FILE *out)
 	return 0;
 }
 
-static int test_pp10(uint8 *data, char *t, int s)
+static int test_pp10(const uint8 *data, char *t, int s)
 {
 	int i;
 	int ntrk, ssize;
@@ -115,7 +113,7 @@ static int test_pp10(uint8 *data, char *t, int s)
 	/* test #2 */
 	ssize = 0;
 	for (i = 0; i < 31; i++) {
-		uint8 *d = data + i * 8;
+		const uint8 *d = data + i * 8;
 		int size = readmem16b(d) << 1;
 		int start = readmem16b(d + 4) << 1;
 		int lsize = readmem16b(d + 6) << 1;
@@ -168,11 +166,10 @@ static int test_pp10(uint8 *data, char *t, int s)
 			ntrk = data[250 + i];
 		}
 	}
-
-	PW_REQUEST_DATA(s, 762 + ntrk * 64);
-
 	ntrk++;
 
+	PW_REQUEST_DATA(s, 762 + ntrk * 256);
+
 	for (i = 0; i < ntrk * 64; i++) {
 		if (data[762 + i * 4] > 0x13) {
 			return -1;
