@@ -12,8 +12,6 @@
  * - Add PP30 support
  */
 
-#include <string.h>
-#include <stdlib.h>
 #include "prowiz.h"
 
 
@@ -31,9 +29,9 @@ static int depack_pp21_pp30(HIO_HANDLE *in, FILE *out, int is_30)
 	int ssize;
 	int tabsize;		/* Reference Table Size */
 
-	memset(ptable, 0, 128);
-	memset(trk, 0, 4 * 128);
-	memset(tptr, 0, 512 * 64 * sizeof (int));
+	memset(ptable, 0, sizeof(ptable));
+	memset(trk, 0, sizeof(trk));
+	memset(tptr, 0, sizeof(tptr));
 
 	pw_write_zero(out, 20);			/* title */
 
@@ -94,10 +92,13 @@ static int depack_pp21_pp30(HIO_HANDLE *in, FILE *out, int is_30)
 
 	/* read "reference Table" */
 	tab = (uint8 *)malloc(tabsize);
-	hio_read(tab, tabsize, 1, in);
+	if (hio_read(tab, tabsize, 1, in) != 1) {
+		free(tab);
+		return -1;
+	}
 
 	for (i = 0; i < numpat; i++) {
-		memset(buf, 0, 1024);
+		memset(buf, 0, sizeof(buf));
 		for (j = 0; j < 64; j++) {
 			uint8 *b = buf + j * 16;
 			memcpy(b, tab + tptr[trk[0][i]][j] * 4, 4);
@@ -126,14 +127,16 @@ static int depack_pp30(HIO_HANDLE *in, FILE *out)
 	return depack_pp21_pp30(in, out, 1);
 }
 
-static int test_pp21(uint8 *data, char *t, int s)
+static int test_pp21(const uint8 *data, char *t, int s)
 {
 	int i;
 	int ssize, tsize, npat, max_ref;
 
+	PW_REQUEST_DATA(s, 762);
+
 	ssize = 0;
 	for (i = 0; i < 31; i++) {
-		uint8 *d = data + i * 8;
+		const uint8 *d = data + i * 8;
 		int len = readmem16b(d) << 1;
 		int start = readmem16b(d + 4) << 1;
 
@@ -171,6 +174,8 @@ static int test_pp21(uint8 *data, char *t, int s)
 	tsize++;
 	tsize <<= 6;
 
+	PW_REQUEST_DATA(s, tsize * 2 + 4 + 762);
+
 	/* test #4  track data value > $4000 ? */
 	max_ref = 0;
 	for (i = 0; i < tsize; i++) {
@@ -194,14 +199,16 @@ static int test_pp21(uint8 *data, char *t, int s)
 }
 
 
-static int test_pp30(uint8 *data, char *t, int s)
+static int test_pp30(const uint8 *data, char *t, int s)
 {
 	int i;
 	int ssize, tsize, npat, max_ref, ref_size;
 
+	PW_REQUEST_DATA(s, 762);
+
 	ssize = 0;
 	for (i = 0; i < 31; i++) {
-		uint8 *d = data + i * 8;
+		const uint8 *d = data + i * 8;
 		int len = readmem16b(d) << 1;
 		int start = readmem16b(d + 4) << 1;
 
@@ -239,6 +246,8 @@ static int test_pp30(uint8 *data, char *t, int s)
 	tsize++;
 	tsize <<= 6;
 
+	PW_REQUEST_DATA(s, (tsize * 2) + 4 + 762);
+
 	/* test #4  track data value *4 ? */
 	max_ref = 0;
 	for (i = 0; i < tsize; i++) {
@@ -266,9 +275,11 @@ static int test_pp30(uint8 *data, char *t, int s)
 
 	ref_size >>= 2;
 
+	PW_REQUEST_DATA(s, (ref_size * 4) + (tsize * 2) + 4 + 762);
+
 	/* test #6  data in reference table ... */
 	for (i = 0; i < ref_size; i++) {
-		uint8 *d = data + tsize + 766 + i * 4;
+		const uint8 *d = data + (tsize * 2) + 766 + i * 4;
 		uint8 fxt = d[2] & 0x0f;
 		uint8 fxp = d[3];
 
@@ -277,8 +288,8 @@ static int test_pp30(uint8 *data, char *t, int s)
 			return -1;
 		}
 
-		/* break > 40 ? */
-		if (fxt == 0x0d && fxp > 0x40) {
+		/* break > 64 (packed decimal) ? */
+		if (fxt == 0x0d && (fxp > 0x64 || (fxp & 0xf) > 9)) {
 			return -1;
 		}
 
