@@ -6,8 +6,6 @@
  * Modified in 2006,2007,2014 by Claudio Matsuoka
  */
 
-#include <string.h>
-#include <stdlib.h>
 #include "prowiz.h"
 
 
@@ -20,8 +18,8 @@ static int depack_pru1 (HIO_HANDLE *in, FILE *out)
 	int ssize = 0;
 	int i, j;
 
-	memset(header, 0, 2048);
-	memset(ptable, 0, 128);
+	memset(header, 0, sizeof(header));
+	memset(ptable, 0, sizeof(ptable));
 
 	/* read and write whole header */
 	hio_read(header, 950, 1, in);
@@ -35,7 +33,7 @@ static int depack_pru1 (HIO_HANDLE *in, FILE *out)
 	/* read and write size of pattern list */
 	write8(out, npat = hio_read8(in));
 
-	memset(header, 0, 2048);
+	memset(header, 0, sizeof(header));
 
 	/* read and write ntk byte and pattern list */
 	hio_read(header, 129, 1, in);
@@ -61,7 +59,7 @@ static int depack_pru1 (HIO_HANDLE *in, FILE *out)
 			header[3] = hio_read8(in);
 
 			/* Sanity check */
-			if (header[1] >= 37) {
+			if (!PTK_IS_VALID_NOTE(header[1])) {
 				return -1;
 			}
 
@@ -84,7 +82,7 @@ static int depack_pru1 (HIO_HANDLE *in, FILE *out)
 	return 0;
 }
 
-static int test_pru1(uint8 *data, char *t, int s)
+static int test_pru1(const uint8 *data, char *t, int s)
 {
 	PW_REQUEST_DATA(s, 1084);
 
