@@ -4,8 +4,6 @@
  *   Modified in 2009,2014 by Claudio Matsuoka
  */
 
-#include <string.h>
-#include <stdlib.h>
 #include "prowiz.h"
 
 
@@ -18,10 +16,11 @@ static int depack_skyt(HIO_HANDLE *in, FILE *out)
 	int i = 0, j = 0, k = 0;
 	int trkval[128][4];
 	int trk_addr;
+	int max_trk;
 	int size, ssize = 0;
 
-	memset(ptable, 0, 128);
-	memset(trkval, 0, 128 * 4);
+	memset(ptable, 0, sizeof(ptable));
+	memset(trkval, 0, sizeof(trkval));
 
 	pw_write_zero(out, 20);			/* write title */
 
@@ -48,9 +47,13 @@ static int depack_skyt(HIO_HANDLE *in, FILE *out)
 	write8(out, 0x7f);			/* write NoiseTracker byte */
 
 	/* read track numbers ... and deduce pattern list */
+	max_trk = 0;
 	for (i = 0; i < pat_pos; i++) {
 		for (j = 0; j < 4; j++) {
 			trkval[i][j] = hio_read16b(in);
+			if (trkval[i][j] > max_trk) {
+					max_trk = trkval[i][j];
+			}
 		}
 	}
 
@@ -68,8 +71,12 @@ static int depack_skyt(HIO_HANDLE *in, FILE *out)
 
 	/* track data */
 	for (i = 0; i < pat_pos; i++) {
-		memset(pat, 0, 1024);
+		memset(pat, 0, sizeof(pat));
 		for (j = 0; j < 4; j++) {
+			/* track 0 is blank and doesn't exist in the file. */
+			if (trkval[i][j] == 0) {
+				continue;
+			}
 			hio_seek(in, trk_addr + ((trkval[i][j] - 1)<<8), SEEK_SET);
 			for (k = 0; k < 64; k++) {
 				int x = k * 16 + j * 4;
@@ -79,6 +86,10 @@ static int depack_skyt(HIO_HANDLE *in, FILE *out)
 				c3 = hio_read8(in);
 				c4 = hio_read8(in);
 
+				if (hio_error(in) || !PTK_IS_VALID_NOTE(c1)) {
+					return -1;
+				}
+
 				pat[x] = (c2 & 0xf0) | ptk_table[c1][0];
 				pat[x + 1] = ptk_table[c1][1];
 				pat[x + 2] = ((c2 << 4) & 0xf0) | c3;
@@ -88,13 +99,18 @@ static int depack_skyt(HIO_HANDLE *in, FILE *out)
 		fwrite(pat, 1024, 1, out);
 	}
 
+	/* skip to the end of the tracks/the start of the sample data. */
+	if (hio_seek(in, trk_addr + (max_trk << 8), SEEK_SET) < 0) {
+		return -1;
+	}
+
 	/* sample data */
 	pw_move_data(out, in, ssize);
 
 	return 0;
 }
 
-static int test_skyt(uint8 *data, char *t, int s)
+static int test_skyt(const uint8 *data, char *t, int s)
 {
 	int i;
 
