@@ -6,14 +6,12 @@
  * Modified in 2006,2007,2014 by Claudio Matsuoka
  */
 
-#include <string.h>
-#include <stdlib.h>
 #include "prowiz.h"
 
 
 static int depack_tdd(HIO_HANDLE *in, FILE *out)
 {
-	uint8 *tmp;
+	uint8 tmp[1024];
 	uint8 pat[1024];
 	uint8 pmax;
 	int i, j, k;
@@ -21,34 +19,32 @@ static int depack_tdd(HIO_HANDLE *in, FILE *out)
 	int saddr[31];
 	int ssizes[31];
 
-	memset(saddr, 0, 31 * 4);
-	memset(ssizes, 0, 31 * 4);
+	memset(saddr, 0, sizeof(saddr));
+	memset(ssizes, 0, sizeof(ssizes));
 
-	/* write ptk header */
-	pw_write_zero(out, 1080);
-
-	/* read/write pattern list + size and ntk byte */
-	tmp = (uint8 *)malloc(130);
-	memset(tmp, 0, 130);
-	fseek(out, 950, 0);
+	/* read pattern list + size and ntk byte */
 	hio_read(tmp, 130, 1, in);
-	fwrite(tmp, 130, 1, out);
 
 	for (pmax = i = 0; i < 128; i++) {
 		if (tmp[i + 2] > pmax) {
 			pmax = tmp[i + 2];
 		}
 	}
-	free(tmp);
+
+	/* title */
+	pw_write_zero(out, 20);
 
 	/* sample descriptions */
 	for (i = 0; i < 31; i++) {
-		fseek(out, 42 + (i * 30), SEEK_SET);
+		/* sample name */
+		pw_write_zero(out, 22);
+
 		/* sample address */
 		saddr[i] = hio_read32b(in);
 
 		/* read/write size */
 		write16b(out, size = hio_read16b(in));
+		size *= 2;
 		ssize += size;
 		ssizes[i] = size;
 
@@ -59,19 +55,26 @@ static int depack_tdd(HIO_HANDLE *in, FILE *out)
 		write16b(out, hio_read16b(in));	/* read/write replen */
 	}
 
-	/* bypass Samples datas */
-	hio_seek(in, ssize, SEEK_CUR);
+	/* write pattern list + size and ntk byte */
+	fwrite(tmp, 130, 1, out);
 
 	/* write ptk's ID string */
-	fseek(out, 0, SEEK_END);
 	write32b(out, PW_MOD_MAGIC);
 
+	/* bypass Samples datas */
+	if (hio_seek(in, ssize, SEEK_CUR) < 0) {
+		return -1;
+	}
+
 	/* read/write pattern data */
-	tmp = (uint8 *)malloc(1024);
 	for (i = 0; i <= pmax; i++) {
-		memset(tmp, 0, 1024);
-		memset(pat, 0, 1024);
-		hio_read(tmp, 1024, 1, in);
+		memset(tmp, 0, sizeof(tmp));
+		memset(pat, 0, sizeof(pat));
+
+		if (hio_read(tmp, 1, 1024, in) != 1024) {
+			return -1;
+		}
+
 		for (j = 0; j < 64; j++) {
 			for (k = 0; k < 4; k++) {
 				int x = j * 16 + k * 4;
@@ -87,13 +90,16 @@ static int depack_tdd(HIO_HANDLE *in, FILE *out)
 				pat[x + 2] |= (tmp[x] << 4) & 0xf0;
 
 				/* note */
-				pat[x] |= ptk_table[tmp[x + 1] / 2][0];
-				pat[x + 1] = ptk_table[tmp[x + 1] / 2][1];
+				if (PTK_IS_VALID_NOTE(tmp[x + 1] / 2)) {
+					pat[x] |= ptk_table[tmp[x + 1] / 2][0];
+					pat[x + 1] = ptk_table[tmp[x + 1] / 2][1];
+				}
 			}
 		}
-		fwrite(pat, 1024, 1, out);
+		if (fwrite(pat, 1, 1024, out) != 1024) {
+			return -1;
+		}
 	}
-	free(tmp);
 
 	/* Sample data */
 	for (i = 0; i < 31; i++) {
@@ -106,7 +112,7 @@ static int depack_tdd(HIO_HANDLE *in, FILE *out)
 	return 0;
 }
 
-static int test_tdd(uint8 *data, char *t, int s)
+static int test_tdd(const uint8 *data, char *t, int s)
 {
 	int i;
 	int ssize, psize, pdata_ofs;
@@ -116,11 +122,12 @@ static int test_tdd(uint8 *data, char *t, int s)
 	/* test #2 (volumes,sample addresses and whole sample size) */
 	ssize = 0;
 	for (i = 0; i < 31; i++) {
-		uint8 *d = data + i * 14;
+		const uint8 *d = data + i * 14;
 		int addr = readmem32b(d + 130);	/* sample address */
 		int size = readmem16b(d + 134);	/* sample size */
 		int sadr = readmem32b(d + 138);	/* loop start address */
 		int lsiz = readmem16b(d + 142);	/* loop size (replen) */
+		size *= 2;
 
 		/* volume > 40h ? */
 		if (d[137] > 0x40)
@@ -174,7 +181,7 @@ static int test_tdd(uint8 *data, char *t, int s)
 	psize <<= 10;
 
 	/* test end of pattern list */
-	for (i = data[0] + 2; i < 128; i++) {
+	for (i = data[0]; i < 128; i++) {
 		if (data[i + 2] != 0)
 			return -1;
 	}
@@ -192,7 +199,7 @@ static int test_tdd(uint8 *data, char *t, int s)
 	PW_REQUEST_DATA(s, 564 + ssize + psize);
 
 	for (i = 0; i < psize; i += 4) {
-		uint8 *d = data + pdata_ofs + i;
+		const uint8 *d = data + pdata_ofs + i;
 
 		/* sample number > 31 ? */
 		if (d[0] > 0x1f)
@@ -217,7 +224,7 @@ static int test_tdd(uint8 *data, char *t, int s)
 
 	pw_read_title(NULL, t, 0);
 
-	return -1;
+	return 0;
 }
 
 const struct pw_format pw_tdd = {
