@@ -1,14 +1,12 @@
 /*
  * Unic_Tracker.c   Copyright (C) 1997 Asle / ReDoX
- * 
+ *
  * Unic tracked MODs to Protracker
  * both with or without ID Unic files will be converted
  *
  * Modified in 2006,2007,2014 by Claudio Matsuoka
  */
 
-#include <string.h>
-#include <stdlib.h>
 #include "prowiz.h"
 
 #define MAGIC_UNIC	MAGIC4('U','N','I','C')
@@ -109,7 +107,7 @@ static int depack_unic(HIO_HANDLE *in, FILE *out)
 			note = c1 & 0x3f;
 
 			/* Sanity check */
-			if (note >= 37) {
+			if (!PTK_IS_VALID_NOTE(note)) {
 				return -1;
 			}
 
@@ -136,7 +134,7 @@ static int depack_unic(HIO_HANDLE *in, FILE *out)
 	return 0;
 }
 
-static int check_instruments(uint8 *data)
+static int check_instruments(const uint8 *data)
 {
 	int ssize, max_ins;
 	int i;
@@ -144,7 +142,7 @@ static int check_instruments(uint8 *data)
 	ssize = 0;
 	max_ins = 0;
 	for (i = 0; i < 31; i++) {
-		uint8 *d = data + i * 30;
+		const uint8 *d = data + i * 30;
 		int len = readmem16b(d + 42) << 1;
 		int start = readmem16b(d + 46) << 1;
 		int lsize = readmem16b(d + 48) << 1;
@@ -185,7 +183,7 @@ static int check_instruments(uint8 *data)
 	return max_ins;
 }
 
-static int check_pattern_list_size(uint8 *data)
+static int check_pattern_list_size(const uint8 *data)
 {
 	int len, psize;
 	int i;
@@ -216,14 +214,14 @@ static int check_pattern_list_size(uint8 *data)
 	return psize;
 }
 
-static int check_pattern(uint8 *data, int s, int psize, int max_ins, int offset)
+static int check_pattern(const uint8 *data, int s, int psize, int max_ins, int offset)
 {
 	int i;
 
 	PW_REQUEST_DATA(s, offset + psize * 3 + 2);
 
 	for (i = 0; i < psize; i++) {
-		uint8 *d = data + offset + i * 3;
+		const uint8 *d = data + offset + i * 3;
 		int ins;
 
 		/* relative note number + last bit of sample > $34 ? */
@@ -240,7 +238,7 @@ static int check_pattern(uint8 *data, int s, int psize, int max_ins, int offset)
 		if ((d[1] & 0x0F) == 0x0D && d[2] > 0x40)
 			return -1;
 
-		ins = ((d[0] >> 2) & 0x30) | ((d[2] >> 4) & 0x0F);
+		ins = ((d[0] >> 2) & 0x30) | ((d[1] >> 4) & 0x0F);
 
 		if (ins > max_ins)
 			return -1;
@@ -249,7 +247,7 @@ static int check_pattern(uint8 *data, int s, int psize, int max_ins, int offset)
 	return 0;
 }
 
-static int test_unic_id(uint8 *data, char *t, int s)
+static int test_unic_id(const uint8 *data, char *t, int s)
 {
 	int i;
 	int psize, ssize;
@@ -263,7 +261,7 @@ static int test_unic_id(uint8 *data, char *t, int s)
 	/* test 2 */
 	ssize = 0;
 	for (i = 0; i < 31; i++) {
-		uint8 *d = data + i * 30;
+		const uint8 *d = data + i * 30;
 		int size, end;
 
 		size = readmem16b(d + 42) << 1;
@@ -279,7 +277,7 @@ static int test_unic_id(uint8 *data, char *t, int s)
 
 	/* test #3  finetunes & volumes */
 	for (i = 0; i < 31; i++) {
-		uint8 *d = data + i * 30;
+		const uint8 *d = data + i * 30;
 		if ((int8)d[40] < -8 || (int8)d[40] > 7)
 			return -1;
 		if (d[44] != 0 || d[45] > 0x40)
@@ -291,6 +289,8 @@ static int test_unic_id(uint8 *data, char *t, int s)
 	if (psize < 0)
 		return -1;
 
+	PW_REQUEST_DATA(s, psize * 3 + 1084);
+
 	/* test #5 pattern data ... */
 	for (i = 0; i < psize; i++) {
 		/* relative note number + last bit of sample > $34 ? */
@@ -303,7 +303,7 @@ static int test_unic_id(uint8 *data, char *t, int s)
 	return 0;
 }
 
-static int test_unic_emptyid(uint8 *data, char *t, int s)
+static int test_unic_emptyid(const uint8 *data, char *t, int s)
 {
 	int psize, max_ins;
 
@@ -333,7 +333,7 @@ static int test_unic_emptyid(uint8 *data, char *t, int s)
 	return 0;
 }
 
-static int test_unic_noid(uint8 *data, char *t, int s)
+static int test_unic_noid(const uint8 *data, char *t, int s)
 {
 	int i;
 	int psize, max_ins;
