@@ -6,8 +6,6 @@
  * Modified in 2006,2007,2014 by Claudio Matsuoka
  */
 
-#include <string.h>
-#include <stdlib.h>
 #include "prowiz.h"
 
 
@@ -92,7 +90,7 @@ static int depack_unic2(HIO_HANDLE *in, FILE *out)
 			note = c1 & 0x3f;
 
 			/* Sanity check */
-			if (note >= 37) {
+			if (!PTK_IS_VALID_NOTE(note)) {
 				return -1;
 			}
 
@@ -120,7 +118,7 @@ static int depack_unic2(HIO_HANDLE *in, FILE *out)
 	return 0;
 }
 
-static int test_unic2(uint8 *data, char *t, int s)
+static int test_unic2(const uint8 *data, char *t, int s)
 {
 	int i;
 	int len, psize, ssize, max_ins;
@@ -136,7 +134,7 @@ static int test_unic2(uint8 *data, char *t, int s)
 	ssize = 0;
 	max_ins = 0;
 	for (i = 0; i < 31; i++) {
-		uint8 *d = data + i * 30;
+		const uint8 *d = data + i * 30;
 
 		int size = readmem16b(d + 22) << 1;
 		int start = readmem16b(d + 26) << 1;
@@ -192,7 +190,7 @@ static int test_unic2(uint8 *data, char *t, int s)
 	PW_REQUEST_DATA (s, 1060 + psize * 3 + 2);
 
 	for (i = 0; i < psize; i++) {
-		uint8 *d = data + 1060 + i * 3;
+		const uint8 *d = data + 1060 + i * 3;
 		int ins;
 
 		/* relative note number + last bit of sample > $34 ? */
