@@ -6,8 +6,6 @@
  * Modified in 2006,2007,2014 by Claudio Matsuoka
  */
 
-#include <string.h>
-#include <stdlib.h>
 #include "prowiz.h"
 
 
@@ -54,7 +52,11 @@ static int depack_wn(HIO_HANDLE *in, FILE * out)
 			c3 = hio_read8(in);
 			c4 = hio_read8(in);
 
-			write8(out, c1 * 0xf0 | ptk_table[c1 / 2][0]);
+			if (hio_error(in) || !PTK_IS_VALID_NOTE(c1 / 2)) {
+				return -1;
+			}
+
+			write8(out, (c2 & 0xf0) | ptk_table[c1 / 2][0]);
 			write8(out, ptk_table[c1 / 2][1]);
 			write8(out, ((c2 << 4) & 0xf0) | c3);
 			write8(out, c4);
@@ -67,7 +69,7 @@ static int depack_wn(HIO_HANDLE *in, FILE * out)
 	return 0;
 }
 
-static int test_wn(uint8 *data, char *t, int s)
+static int test_wn(const uint8 *data, char *t, int s)
 {
 	PW_REQUEST_DATA(s, 1082);
 
