@@ -6,7 +6,6 @@
  * Modified in 2006,2007,2014 by Claudio Matsuoka
  */
 
-#include <string.h>
 #include "prowiz.h"
 
 #define SMP_DESC_ADDRESS 0x206
@@ -25,8 +24,8 @@ static int depack_xann(HIO_HANDLE *in, FILE *out)
 	int size, ssize = 0;
 	int lsize;
 
-	memset(ptable, 0, 128);
-	memset(pdata, 0, 1025);
+	memset(ptable, 0, sizeof(ptable));
+	memset(pdata, 0, sizeof(pdata));
 
 	pw_write_zero(out, 20);			/* title */
 
@@ -84,7 +83,7 @@ static int depack_xann(HIO_HANDLE *in, FILE *out)
 			fxt = hio_read8(in);
 			fxp = hio_read8(in);
 
-			if (hio_error(in) || note >= 74) {
+			if (hio_error(in) || !PTK_IS_VALID_NOTE(note >> 1)) {
 				return -1;
 			}
 
@@ -203,7 +202,7 @@ static int depack_xann(HIO_HANDLE *in, FILE *out)
 	return 0;
 }
 
-static int test_xann(uint8 *data, char *t, int s)
+static int test_xann(const uint8 *data, char *t, int s)
 {
 	int i;
 
