@@ -6,8 +6,6 @@
  * Modified in 2006,2007,2014 by Claudio Matsuoka
  */
 
-#include <string.h>
-#include <stdlib.h>
 #include "prowiz.h"
 
 
@@ -27,9 +25,9 @@ static int depack_zen(HIO_HANDLE *in, FILE *out)
 	int sdata_addr = 999999l;
 	int i, j, k;
 
-	memset(paddr, 0, 128 * 4);
-	memset(paddr2, 0, 128 * 4);
-	memset(ptable, 0, 128);
+	memset(paddr, 0, sizeof(paddr));
+	memset(paddr2, 0, sizeof(paddr2));
+	memset(ptable, 0, sizeof(ptable));
 
 	ptable_addr = hio_read32b(in);	/* read pattern table address */
 	pat_max = hio_read8(in);	/* read patmax */
@@ -106,7 +104,7 @@ static int depack_zen(HIO_HANDLE *in, FILE *out)
 	/* pattern data */
 	/*printf ( "converting pattern datas " ); */
 	for (i = 0; i <= pat_max; i++) {
-		memset(pat, 0, 1024);
+		memset(pat, 0, sizeof(pat));
 		hio_seek(in, paddr2[i], SEEK_SET);
 		for (j = 0; j < 256; j++) {
 			uint8 *p;
@@ -117,6 +115,11 @@ static int depack_zen(HIO_HANDLE *in, FILE *out)
 			c4 = hio_read8(in);
 
 			note = (c2 & 0x7f) / 2;
+
+			if (hio_error(in) || !PTK_IS_VALID_NOTE(note)) {
+				return -1;
+			}
+
 			fxp = c4;
 			ins = ((c2 << 4) & 0x10) | ((c3 >> 4) & 0x0f);
 			fxt = c3 & 0x0f;
@@ -140,7 +143,7 @@ static int depack_zen(HIO_HANDLE *in, FILE *out)
 	return 0;
 }
 
-static int test_zen(uint8 *data, char *t, int s)
+static int test_zen(const uint8 *data, char *t, int s)
 {
 	int i;
 	int len, pat_ofs;
@@ -153,7 +156,7 @@ static int test_zen(uint8 *data, char *t, int s)
 		return -1;
 
 	for (i = 0; i < 31; i++) {
-		uint8 *d = data + 16 * i;
+		const uint8 *d = data + 16 * i;
 		if (d[9] > 0x40)
 			return -1;
 
