@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2016 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2021 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -20,7 +20,6 @@
  * THE SOFTWARE.
  */
 
-#include <stdio.h>
 #include "loader.h"
 #include "mod.h"
 #include "iff.h"
@@ -56,7 +55,7 @@ static int pt3_test(HIO_HANDLE *f, char *t, const int start)
 	hio_read32b(f);	/* skip size */
 
 	hio_seek(f, 10, SEEK_CUR);
-	
+
 	if (hio_read32b(f) == MAGIC_INFO) {
 		hio_read32b(f);	/* skip size */
 		libxmp_read_title(f, t, 32);
@@ -76,14 +75,23 @@ static int pt3_test(HIO_HANDLE *f, char *t, const int start)
 #define PT3_FLAG_16BIT	0x0040	/* 8 bit samples if not set */
 #define PT3_FLAG_RAWPAT	0x0080	/* Packed patterns if not set */
 
+struct local_data {
+	int has_ptdt;
+};
 
 static int get_info(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 {
 	struct xmp_module *mod = &m->mod;
+	struct local_data *data = (struct local_data *)parm;
 	/* int flags; */
 	/* int day, month, year, hour, min, sec;
 	int dhour, dmin, dsec; */
 
+	/* Sanity check */
+	if(data->has_ptdt) {
+		return -1;
+	}
+
 	hio_read(mod->name, 1, 32, f);
 	mod->ins = hio_read16b(f);
 	mod->len = hio_read16b(f);
@@ -124,6 +132,14 @@ static int get_cmnt(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 
 static int get_ptdt(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 {
+	struct local_data *data = (struct local_data *)parm;
+
+	/* Sanity check */
+	if(data->has_ptdt) {
+		return -1;
+	}
+	data->has_ptdt = 1;
+
 	ptdt_load(m, f, 0);
 
 	return 0;
@@ -132,11 +148,14 @@ static int get_ptdt(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 static int pt3_load(struct module_data *m, HIO_HANDLE *f, const int start)
 {
 	iff_handle handle;
+	struct local_data data;
 	char buf[20];
 	int ret;
 
 	LOAD_INIT();
 
+	memset(&data, 0, sizeof(struct local_data));
+
 	hio_read32b(f);		/* FORM */
 	hio_read32b(f);		/* size */
 	hio_read32b(f);		/* MODL */
@@ -161,7 +180,7 @@ static int pt3_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	libxmp_iff_set_quirk(handle, IFF_FULL_CHUNK_SIZE);
 
 	/* Load IFF chunks */
-	if (libxmp_iff_load(handle, m, f, NULL) < 0) {
+	if (libxmp_iff_load(handle, m, f, &data) < 0) {
 		libxmp_iff_release(handle);
 		return -1;
 	}
@@ -184,9 +203,9 @@ static int ptdt_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	struct mod_header mh;
 	uint8 mod_event[4];
 
-	hio_read(&mh.name, 20, 1, f);
+	hio_read(mh.name, 20, 1, f);
 	for (i = 0; i < 31; i++) {
-		hio_read(&mh.ins[i].name, 22, 1, f);
+		hio_read(mh.ins[i].name, 22, 1, f);
 		mh.ins[i].size = hio_read16b(f);
 		mh.ins[i].finetune = hio_read8(f);
 		mh.ins[i].volume = hio_read8(f);
@@ -195,8 +214,11 @@ static int ptdt_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	}
 	mh.len = hio_read8(f);
 	mh.restart = hio_read8(f);
-	hio_read(&mh.order, 128, 1, f);
-	hio_read(&mh.magic, 4, 1, f);
+	if (hio_read(mh.order, 128, 1, f) < 1) {
+		D_(D_CRIT "read error at order list");
+		return -1;
+	}
+	hio_read(mh.magic, 4, 1, f);
 
 	mod->ins = 31;
 	mod->smp = mod->ins;
@@ -257,7 +279,10 @@ static int ptdt_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
 		for (j = 0; j < (64 * 4); j++) {
 			event = &EVENT(i, j % 4, j / 4);
-			hio_read(mod_event, 1, 4, f);
+			if (hio_read(mod_event, 1, 4, f) < 4) {
+				D_(D_CRIT "read error at pat %d", i);
+				return -1;
+			}
 			libxmp_decode_protracker_event(event, mod_event);
 		}
 	}
