@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2016 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2021 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -184,18 +184,18 @@ static void unroll_loop(struct xmp_sample *xxs)
 	if (xxs->flg & XMP_SAMPLE_16BIT) {
 		s16 += start;
 		for (i = 0; i < loop_size; i++) {
-			*(s16 + i) = *(s16 - i - 1);	
+			*(s16 + i) = *(s16 - i - 1);
 		}
 	} else {
 		s8 += start;
 		for (i = 0; i < loop_size; i++) {
-			*(s8 + i) = *(s8 - i - 1);	
+			*(s8 + i) = *(s8 - i - 1);
 		}
 	}
 }
 
 
-int libxmp_load_sample(struct module_data *m, HIO_HANDLE *f, int flags, struct xmp_sample *xxs, void *buffer)
+int libxmp_load_sample(struct module_data *m, HIO_HANDLE *f, int flags, struct xmp_sample *xxs, const void *buffer)
 {
 	int bytelen, extralen, unroll_extralen, i;
 
@@ -225,6 +225,27 @@ int libxmp_load_sample(struct module_data *m, HIO_HANDLE *f, int flags, struct x
 		return 0;
 	}
 
+	/* If this sample starts at or after EOF, skip it entirely.
+	 */
+	if (~flags & SAMPLE_FLAG_NOLOAD) {
+		long file_pos, file_len;
+		if (!f) {
+			return 0;
+		}
+		file_pos = hio_tell(f);
+		file_len = hio_size(f);
+		if (file_pos >= file_len) {
+			D_(D_WARN "ignoring sample at EOF");
+			return 0;
+		}
+		/* If this sample goes past EOF, truncate it. */
+		if (file_pos + xxs->len > file_len && (~flags & SAMPLE_FLAG_ADPCM)) {
+			D_(D_WARN "sample would extend %ld bytes past EOF; truncating to %ld",
+				file_pos + xxs->len - file_len, file_len - file_pos);
+			xxs->len = file_len - file_pos;
+		}
+	}
+
 	/* Loop parameters sanity check
 	 */
 	if (xxs->lps < 0) {
@@ -363,7 +384,7 @@ int libxmp_load_sample(struct module_data *m, HIO_HANDLE *f, int flags, struct x
 		unroll_loop(xxs);
 		bytelen += unroll_extralen;
 	}
-	
+
 	/* Add extra samples at end */
 	if (xxs->flg & XMP_SAMPLE_16BIT) {
 		for (i = 0; i < 8; i++) {
@@ -409,8 +430,16 @@ int libxmp_load_sample(struct module_data *m, HIO_HANDLE *f, int flags, struct x
 
 #ifndef LIBXMP_CORE_PLAYER
     err2:
-	free(xxs->data - 4);
+	libxmp_free_sample(xxs);
 #endif
     err:
 	return -1;
 }
+
+void libxmp_free_sample(struct xmp_sample *s)
+{
+    if (s->data) {
+	free(s->data - 4);
+	s->data = NULL;		/* prevent double free in PCM load error */
+    }
+}
