@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2016 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2021 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -20,6 +20,7 @@
  * THE SOFTWARE.
  */
 
+#include <limits.h>
 #include "loader.h"
 #include "period.h"
 
@@ -41,6 +42,26 @@ struct stm_instrument_header {
 	uint16 paralen;		/* Length in paragraphs */
 };
 
+/* v1 format header based on disassembled ST2 */
+struct stm_file_subheader_v1 {
+	uint16 insnum;		/* Number of instruments */
+	uint16 ordnum;		/* Number of orders */
+	uint16 patnum;		/* Number of patterns */
+	uint16 srate;		/* Sample rate? */
+	uint8 tempo;		/* Playback tempo */
+	uint8 channels;		/* Number of channels */
+	uint16 psize;		/* Pattern size */
+	uint16 rsvd2;		/* Reserved */
+	uint16 skip;		/* Bytes to skip */
+};
+
+struct stm_file_subheader_v2 {
+	uint8 tempo;		/* Playback tempo */
+	uint8 patterns;		/* Number of patterns */
+	uint8 gvol;		/* Global volume */
+	uint8 rsvd2[13];	/* Reserved */
+};
+
 struct stm_file_header {
 	uint8 name[20];		/* ASCIIZ song name */
 	uint8 magic[8];		/* '!Scream!' */
@@ -48,11 +69,11 @@ struct stm_file_header {
 	uint8 type;		/* 1=song, 2=module */
 	uint8 vermaj;		/* Major version number */
 	uint8 vermin;		/* Minor version number */
-	uint8 tempo;		/* Playback tempo */
-	uint8 patterns;		/* Number of patterns */
-	uint8 gvol;		/* Global volume */
-	uint8 rsvd2[13];	/* Reserved */
-	struct stm_instrument_header ins[31];
+	union {
+		struct stm_file_subheader_v1 v1;
+		struct stm_file_subheader_v2 v2;
+	} sub;
+	struct stm_instrument_header ins[32];
 };
 
 static int stm_test(HIO_HANDLE *, char *, const int);
@@ -66,20 +87,25 @@ const struct format_loader libxmp_loader_stm = {
 
 static int stm_test(HIO_HANDLE * f, char *t, const int start)
 {
-	char buf[8];
+	uint8 buf[8];
 
 	hio_seek(f, start + 20, SEEK_SET);
 	if (hio_read(buf, 1, 8, f) < 8)
 		return -1;
-	if (memcmp(buf, "!Scream!", 8) && memcmp(buf, "BMOD2STM", 8))
+
+	if (libxmp_test_name(buf, 8))	/* Tracker name should be ASCII */
 		return -1;
 
-	hio_read8(f);
-
-	if (hio_read8(f) != STM_TYPE_MODULE)
+	if (hio_read8(f) != 0x1a)
 		return -1;
 
-	if (hio_read8(f) < 1)	/* We don't want STX files */
+	if (hio_read8(f) > STM_TYPE_MODULE)
+		return -1;
+
+	hio_seek(f, start + 60, SEEK_SET);
+	if (hio_read(buf, 1, 4, f) < 4)
+		return -1;
+	if (!memcmp(buf, "SCRM", 4))	/* We don't want STX files */
 		return -1;
 
 	hio_seek(f, start + 0, SEEK_SET);
@@ -99,8 +125,7 @@ static int stm_test(HIO_HANDLE * f, char *t, const int start)
  * this in some songs to give the lead some character. However, when played
  * in ModPlug Tracker, this effect doesn't work the way it did back then.
  */
-
-static const uint8 fx[] = {
+static const uint8 fx[16] = {
 	FX_NONE,
 	FX_SPEED,	/* A - Set tempo to [INFO]. 60 normal. */
 	FX_JUMP,	/* B - Break pattern and jmp to order [INFO] */
@@ -111,33 +136,87 @@ static const uint8 fx[] = {
 	FX_TONEPORTA,	/* G - Slide to the note specified at speed [INFO] */
 	FX_VIBRATO,	/* H - Vibrato; Hi-nibble, speed. Lo-nibble, size */
 	FX_TREMOR,	/* I - Tremor; Hi-nibble, ontime. Lo-nibble, offtime */
-	FX_ARPEGGIO	/* J - Arpeggio */
+	FX_ARPEGGIO,	/* J - Arpeggio */
+	FX_NONE,
+	FX_NONE,
+	FX_NONE,
+	FX_NONE,
+	FX_NONE
 };
 
 static int stm_load(struct module_data *m, HIO_HANDLE * f, const int start)
 {
 	struct xmp_module *mod = &m->mod;
-	int i, j;
 	struct xmp_event *event;
 	struct stm_file_header sfh;
 	uint8 b;
-	int bmod2stm = 0;
+	uint16 version;
+	int i, j;
 
 	LOAD_INIT();
 
-	hio_read(&sfh.name, 20, 1, f);	/* ASCIIZ song name */
-	hio_read(&sfh.magic, 8, 1, f);	/* '!Scream!' */
+	hio_read(sfh.name, 20, 1, f);	/* ASCIIZ song name */
+	hio_read(sfh.magic, 8, 1, f);	/* '!Scream!' */
 	sfh.rsvd1 = hio_read8(f);	/* '\x1a' */
 	sfh.type = hio_read8(f);	/* 1=song, 2=module */
 	sfh.vermaj = hio_read8(f);	/* Major version number */
 	sfh.vermin = hio_read8(f);	/* Minor version number */
-	sfh.tempo = hio_read8(f);	/* Playback tempo */
-	sfh.patterns = hio_read8(f);	/* Number of patterns */
-	sfh.gvol = hio_read8(f);	/* Global volume */
-	hio_read(&sfh.rsvd2, 13, 1, f);	/* Reserved */
+	version = (100 * sfh.vermaj) + sfh.vermin;
 
-	for (i = 0; i < 31; i++) {
-		hio_read(&sfh.ins[i].name, 12, 1, f);	/* Instrument name */
+	if (version != 110 &&
+	    version != 200 && version != 210 &&
+	    version != 220 && version != 221) {
+		D_(D_CRIT "Unknown version: %d", version);
+		return -1;
+	}
+
+	// TODO: improve robustness of the loader against bad parameters
+
+	if (version >= 200) {
+		sfh.sub.v2.tempo = hio_read8(f);	/* Playback tempo */
+		sfh.sub.v2.patterns = hio_read8(f);	/* Number of patterns */
+		sfh.sub.v2.gvol = hio_read8(f);		/* Global volume */
+		hio_read(sfh.sub.v2.rsvd2, 13, 1, f);	/* Reserved */
+		mod->chn = 4;
+		mod->pat = sfh.sub.v2.patterns;
+		mod->spd = (version < 221) ? LSN(sfh.sub.v2.tempo / 10) : MSN(sfh.sub.v2.tempo);
+		mod->ins = 31;
+		mod->len = (version == 200) ? 64 : 128;
+	} else {
+		if ((sfh.sub.v1.insnum = hio_read16l(f)) > 32) {	/* Number of instruments */
+			D_(D_CRIT "Wrong number of instruments: %d (max 32)", sfh.sub.v1.insnum);
+			return -1;
+		}
+		if ((sfh.sub.v1.ordnum = hio_read16l(f)) > XMP_MAX_MOD_LENGTH) {	/* Number of orders */
+			D_(D_CRIT "Wrong number of orders: %d (max %d)", sfh.sub.v1.ordnum, XMP_MAX_MOD_LENGTH);
+			return -1;
+		}
+		if ((sfh.sub.v1.patnum = hio_read16l(f)) > XMP_MAX_MOD_LENGTH) {	/* Number of patterns */
+			D_(D_CRIT "Wrong number of patterns: %d (max %d)", sfh.sub.v1.patnum, XMP_MAX_MOD_LENGTH);
+			return -1;
+		}
+		sfh.sub.v1.srate = hio_read16l(f);	/* Sample rate? */
+		sfh.sub.v1.tempo = hio_read8(f);	/* Playback tempo */
+		if ((sfh.sub.v1.channels = hio_read8(f)) != 4) {	/* Number of channels */
+			D_(D_CRIT "Wrong number of sound channels: %d", sfh.sub.v1.channels);
+			return -1;
+		}
+		if ((sfh.sub.v1.psize = hio_read16l(f)) != 64) {	/* Pattern size */
+			D_(D_CRIT "Wrong number of rows per pattern: %d", sfh.sub.v1.psize);
+			return -1;
+		}
+		sfh.sub.v1.rsvd2 = hio_read16l(f);	/* Reserved */
+		sfh.sub.v1.skip = hio_read16l(f);	/* Bytes to skip */
+		hio_seek(f, sfh.sub.v1.skip, SEEK_CUR);	/* Skip bytes */
+		mod->chn = sfh.sub.v1.channels;
+		mod->pat = sfh.sub.v1.patnum;
+		mod->spd = (version != 100) ? LSN(sfh.sub.v1.tempo / 10) : LSN(sfh.sub.v1.tempo);
+		mod->ins = sfh.sub.v1.insnum;
+		mod->len = sfh.sub.v1.ordnum;
+	}
+
+	for (i = 0; i < mod->ins; i++) {
+		hio_read(sfh.ins[i].name, 12, 1, f);	/* Instrument name */
 		sfh.ins[i].id = hio_read8(f);		/* Id=0 */
 		sfh.ins[i].idisk = hio_read8(f);	/* Instrument disk */
 		sfh.ins[i].rsvd1 = hio_read16l(f);	/* Reserved */
@@ -155,25 +234,18 @@ static int stm_load(struct module_data *m, HIO_HANDLE * f, const int start)
 		return -1;
 	}
 
-	if (!strncmp((char *)sfh.magic, "BMOD2STM", 8))
-		bmod2stm = 1;
-
-	mod->chn = 4;
-	mod->pat = sfh.patterns;
 	mod->trk = mod->pat * mod->chn;
-	mod->spd = MSN(sfh.tempo);
-	mod->ins = 31;
 	mod->smp = mod->ins;
 	m->c4rate = C4_NTSC_RATE;
 
 	libxmp_copy_adjust(mod->name, sfh.name, 20);
 
-	if (bmod2stm) {
-		snprintf(mod->type, XMP_NAME_SIZE, "BMOD2STM STM");
-	} else {
-		snprintf(mod->type, XMP_NAME_SIZE, "Scream Tracker %d.%02d STM",
-			 sfh.vermaj, sfh.vermin);
-	}
+	if (!sfh.magic[0] || !strncmp((char *)sfh.magic, "PCSTV", 5) || !strncmp((char *)sfh.magic, "!Scream!", 8))
+		libxmp_set_type(m, "Scream Tracker %d.%02d", sfh.vermaj, sfh.vermin);
+	else if (!strncmp((char *)sfh.magic, "SWavePro", 8))
+		libxmp_set_type(m, "SoundWave Pro %d.%02d", sfh.vermaj, sfh.vermin);
+	else
+		libxmp_copy_adjust(mod->type, sfh.magic, 8);
 
 	MODULE_INFO();
 
@@ -210,9 +282,10 @@ static int stm_load(struct module_data *m, HIO_HANDLE * f, const int start)
 			      &mod->xxi[i].sub[0].fin);
 	}
 
-	hio_read(mod->xxo, 1, 128, f);
+	if (hio_read(mod->xxo, 1, mod->len, f) < mod->len)
+		return -1;
 
-	for (i = 0; i < 128; i++)
+	for (i = 0; i < mod->len; i++)
 		if (mod->xxo[i] >= mod->pat)
 			break;
 
@@ -230,6 +303,9 @@ static int stm_load(struct module_data *m, HIO_HANDLE * f, const int start)
 		if (libxmp_alloc_pattern_tracks(mod, i, 64) < 0)
 			return -1;
 
+		if (hio_error(f))
+			return -1;
+
 		for (j = 0; j < 64 * mod->chn; j++) {
 			event = &EVENT(i, j % mod->chn, j / mod->chn);
 			b = hio_read8(f);
@@ -237,26 +313,37 @@ static int stm_load(struct module_data *m, HIO_HANDLE * f, const int start)
 			switch (b) {
 			case 251:
 			case 252:
+				break; /* Empty note */
 			case 253:
-				break;
-			case 255:
+				event->note = XMP_KEY_OFF;
+				break; /* Key off */
 			default:
-				event->note = b == 255 ? 0 :
-					1 + LSN(b) + 12 * (3 + MSN(b));
+				if (b == 254)
+					event->note = XMP_KEY_OFF;
+				else if (b == 255)
+					event->note = 0;
+				else
+					event->note = 1 + LSN(b) + 12 * (3 + MSN(b));
+
 				b = hio_read8(f);
 				event->vol = b & 0x07;
 				event->ins = (b & 0xf8) >> 3;
+
 				b = hio_read8(f);
 				event->vol += (b & 0xf0) >> 1;
-				if (event->vol > 0x40)
-					event->vol = 0;
-				else
-					event->vol++;
+				if (version >= 200) {
+					event->vol = (event->vol > 0x40) ? 0 : event->vol + 1;
+				} else {
+					if (event->vol > 0) {
+						event->vol = (event->vol > 0x40) ? 1 : event->vol + 1;
+					}
+				}
+
 				event->fxt = fx[LSN(b)];
 				event->fxp = hio_read8(f);
 				switch (event->fxt) {
 				case FX_SPEED:
-					event->fxp = MSN(event->fxp);
+					event->fxp = (version < 221) ? LSN(event->fxp / 10) : MSN(event->fxp);
 					break;
 				case FX_NONE:
 					event->fxp = event->fxt = 0;
@@ -270,11 +357,36 @@ static int stm_load(struct module_data *m, HIO_HANDLE * f, const int start)
 	D_(D_INFO "Stored samples: %d", mod->smp);
 
 	for (i = 0; i < mod->ins; i++) {
-		if (mod->xxs[i].len > 1) {
+		if (!sfh.ins[i].volume || !sfh.ins[i].length) {
+			mod->xxi[i].nsm = 0;
+			continue;
+		}
+
+		if (sfh.type == STM_TYPE_SONG) {
+			HIO_HANDLE *s;
+			char sn[PATH_MAX];
+			char tmpname[32];
+			const char *instname = mod->xxi[i].name;
+
+			if (!instname[0] || !m->dirname)
+				continue;
+
+			if (libxmp_copy_name_for_fopen(tmpname, instname, 32))
+				continue;
+
+			snprintf(sn, PATH_MAX, "%s%s", m->dirname, tmpname);
+
+			if ((s = hio_open(sn, "rb"))) {
+				if (libxmp_load_sample(m, s, SAMPLE_FLAG_UNS, &mod->xxs[i], NULL) < 0) {
+					hio_close(s);
+					return -1;
+				}
+				hio_close(s);
+			}
+		} else {
+			hio_seek(f, start + (sfh.ins[i].rsvd1 << 4), SEEK_SET);
 			if (libxmp_load_sample(m, f, 0, &mod->xxs[i], NULL) < 0)
 				return -1;
-		} else {
-			mod->xxi[i].nsm = 0;
 		}
 	}
 
