@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2016 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2021 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -20,8 +20,6 @@
  * THE SOFTWARE.
  */
 
-#include <stdlib.h>
-#include <string.h>
 #include <math.h>
 #include "common.h"
 #include "virtual.h"
@@ -47,7 +45,7 @@
 #define LIM16_HI	 32767
 #define LIM16_LO	-32768
 
-#define MIX_FN(x) void libxmp_mix_##x(struct mixer_voice *, int *, int, int, int, int, int, int, int)
+#define MIX_FN(x) void libxmp_mix_##x(struct mixer_voice *, int32 *, int, int, int, int, int, int, int)
 
 MIX_FN(mono_8bit_nearest);
 MIX_FN(mono_8bit_linear);
@@ -87,9 +85,9 @@ MIX_FN(stereo_a500_filter);
  * bit 2: 0=unfiltered, 1=filtered
  */
 
-typedef void (*mixer_set[])(struct mixer_voice *, int *, int, int, int, int, int, int, int);
+typedef void (*MIX_FP) (struct mixer_voice *, int32 *, int, int, int, int, int, int, int);
 
-static mixer_set nearest_mixers = {
+static MIX_FP nearest_mixers[] = {
 	libxmp_mix_mono_8bit_nearest,
 	libxmp_mix_mono_16bit_nearest,
 	libxmp_mix_stereo_8bit_nearest,
@@ -103,7 +101,7 @@ static mixer_set nearest_mixers = {
 #endif
 };
 
-static mixer_set linear_mixers = {
+static MIX_FP linear_mixers[] = {
 	libxmp_mix_mono_8bit_linear,
 	libxmp_mix_mono_16bit_linear,
 	libxmp_mix_stereo_8bit_linear,
@@ -117,7 +115,7 @@ static mixer_set linear_mixers = {
 #endif
 };
 
-static mixer_set spline_mixers = {
+static MIX_FP spline_mixers[] = {
 	libxmp_mix_mono_8bit_spline,
 	libxmp_mix_mono_16bit_spline,
 	libxmp_mix_stereo_8bit_spline,
@@ -132,7 +130,7 @@ static mixer_set spline_mixers = {
 };
 
 #ifdef LIBXMP_PAULA_SIMULATOR
-static mixer_set a500_mixers = {
+static MIX_FP a500_mixers[] = {
 	libxmp_mix_mono_a500,
 	NULL,
 	libxmp_mix_stereo_a500,
@@ -144,7 +142,7 @@ static mixer_set a500_mixers = {
 };
 
 
-static mixer_set a500led_mixers = {
+static MIX_FP a500led_mixers[] = {
 	libxmp_mix_mono_a500_filter,
 	NULL,
 	libxmp_mix_stereo_a500_filter,
@@ -341,30 +339,30 @@ void libxmp_mixer_softmixer(struct context_data *ctx)
 	int prev_l, prev_r = 0;
 	int lps, lpe;
 	int32 *buf_pos;
-	void (*mix_fn)(struct mixer_voice *, int *, int, int, int, int, int, int, int);
-	mixer_set *mixers;
+	MIX_FP  mix_fn;
+	MIX_FP *mixerset;
 
 	switch (s->interp) {
 	case XMP_INTERP_NEAREST:
-		mixers = &nearest_mixers;
+		mixerset = nearest_mixers;
 		break;
 	case XMP_INTERP_LINEAR:
-		mixers = &linear_mixers;
+		mixerset = linear_mixers;
 		break;
 	case XMP_INTERP_SPLINE:
-		mixers = &spline_mixers;
+		mixerset = spline_mixers;
 		break;
 	default:
-		mixers = &linear_mixers;
+		mixerset = linear_mixers;
 	}
 
 #ifdef LIBXMP_PAULA_SIMULATOR
 	if (p->flags & XMP_FLAGS_A500) {
 		if (IS_AMIGA_MOD()) {
 			if (p->filter) {
-				mixers = &a500led_mixers;
+				mixerset = a500led_mixers;
 			} else {
-				mixers = &a500_mixers;
+				mixerset = a500_mixers;
 			}
 		}
 	}
@@ -373,7 +371,7 @@ void libxmp_mixer_softmixer(struct context_data *ctx)
 	libxmp_mixer_prepare(ctx);
 
 	for (voc = 0; voc < p->virt.maxvoc; voc++) {
-		int c5spd;
+		int c5spd, rampsize, delta_l, delta_r;
 
 		vi = &p->virt.voice_array[voc];
 
@@ -447,9 +445,9 @@ void libxmp_mixer_softmixer(struct context_data *ctx)
 #endif
 		}
 
-		int rampsize = s->ticksize >> ANTICLICK_SHIFT;
-		int delta_l = (vol_l - vi->old_vl) / rampsize;
-		int delta_r = (vol_r - vi->old_vr) / rampsize;
+		rampsize = s->ticksize >> ANTICLICK_SHIFT;
+		delta_l = (vol_l - vi->old_vl) / rampsize;
+		delta_r = (vol_r - vi->old_vr) / rampsize;
 
 		usmp = 0;
 		for (size = s->ticksize; size > 0; ) {
@@ -479,7 +477,7 @@ void libxmp_mixer_softmixer(struct context_data *ctx)
 
 			if (vi->vol) {
 				int mix_size = samples;
-				int mixer = vi->fidx & FIDX_FLAGMASK;
+				int mixer_id = vi->fidx & FIDX_FLAGMASK;
 
 				if (~s->format & XMP_FORMAT_MONO) {
 					mix_size *= 2;
@@ -499,11 +497,11 @@ void libxmp_mixer_softmixer(struct context_data *ctx)
 				/* See OpenMPT env-flt-max.it */
 				if (vi->filter.cutoff >= 0xfe &&
                                     vi->filter.resonance == 0) {
-					mixer &= ~FLAG_FILTER;
+					mixer_id &= ~FLAG_FILTER;
 				}
 #endif
 
-				mix_fn = (*mixers)[mixer];
+				mix_fn = mixerset[mixer_id];
 
 				/* Call the output handler */
 				if (samples > 0 && vi->sptr != NULL) {
@@ -803,11 +801,11 @@ int libxmp_mixer_on(struct context_data *ctx, int rate, int format, int c4rate)
 {
 	struct mixer_data *s = &ctx->s;
 
-	s->buffer = calloc(2, XMP_MAX_FRAMESIZE);
+	s->buffer = (char *) calloc(2, XMP_MAX_FRAMESIZE);
 	if (s->buffer == NULL)
 		goto err;
 
-	s->buf32 = calloc(sizeof(int), XMP_MAX_FRAMESIZE);
+	s->buf32 = (int32 *) calloc(sizeof(int32), XMP_MAX_FRAMESIZE);
 	if (s->buf32 == NULL)
 		goto err1;
 
@@ -815,7 +813,7 @@ int libxmp_mixer_on(struct context_data *ctx, int rate, int format, int c4rate)
 	s->format = format;
 	s->amplify = DEFAULT_AMPLIFY;
 	s->mix = DEFAULT_MIX;
-	/* s->pbase = C4_PERIOD * c4rate / s->freq; */
+	/* s->pbase = C4_PERIOD * c4rate / s->freq; */(void) c4rate;
 	s->interp = XMP_INTERP_LINEAR;	/* default interpolation type */
 	s->dsp = XMP_DSP_LOWPASS;	/* enable filters by default */
 	/* s->numvoc = SMIX_NUMVOC; */
@@ -825,6 +823,7 @@ int libxmp_mixer_on(struct context_data *ctx, int rate, int format, int c4rate)
 
     err1:
 	free(s->buffer);
+	s->buffer = NULL;
     err:
 	return -1;
 }
