@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2016 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2021 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -19,9 +19,10 @@
  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  * THE SOFTWARE.
  */
+#ifndef _GNU_SOURCE
+#define _GNU_SOURCE
+#endif
 
-#include <stdio.h>
-#include <string.h>
 #include "common.h"
 #include "period.h"
 
@@ -131,11 +132,16 @@ static uint16 pt_period_table[16][36] = {
 };
 #endif
 
-#ifdef _MSC_VER
-static inline double round(double val)
+#ifndef M_LN2
+#define M_LN2	0.69314718055994530942
+#endif
+#if !defined(HAVE_ROUND) || defined(_MSC_VER) || defined(__WATCOMC__) || defined(__DJGPP__)
+static inline double libxmp_round(double val)
 {
-	return floor(val + 0.5);
+	return (val >= 0.0)? floor(val + 0.5) : ceil(val - 0.5);
 }
+#else
+#define libxmp_round round
 #endif
 
 #ifdef LIBXMP_PAULA_SIMULATOR
@@ -213,7 +219,7 @@ int libxmp_period_to_note(int p)
 		return 0;
 	}
 
-	return round(12.0 * log(PERIOD_BASE / p) / M_LN2) + 1;
+	return libxmp_round(12.0 * log(PERIOD_BASE / p) / M_LN2) + 1;
 }
 
 /* Get pitchbend from base note and amiga period */
@@ -231,11 +237,11 @@ int libxmp_period_to_bend(struct context_data *ctx, double p, int n, double adj)
 		return 100 * (8 * (((240 - n) << 4) - p));
 	case PERIOD_CSPD:
 		d = libxmp_note_to_period(ctx, n, 0, adj);
-		return round(100.0 * (1536.0 / M_LN2) * log(p / d));
+		return libxmp_round(100.0 * (1536.0 / M_LN2) * log(p / d));
 	default:
 		/* Amiga */
 		d = libxmp_note_to_period(ctx, n, 0, adj);
-		return round(100.0 * (1536.0 / M_LN2) * log(d / p));
+		return libxmp_round(100.0 * (1536.0 / M_LN2) * log(d / p));
 	}
 }
 
