@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2016 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2021 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -37,8 +37,6 @@
  * Claudio's fix: implementing effect K
  */
 
-#include <stdlib.h>
-#include <string.h>
 #include "virtual.h"
 #include "period.h"
 #include "effects.h"
@@ -336,6 +334,23 @@ static int ft2_arpeggio(struct context_data *ctx, struct channel_data *xc)
 	return xc->arpeggio.val[i % 3];
 }
 
+static int arpeggio(struct context_data *ctx, struct channel_data *xc)
+{
+	struct module_data *m = &ctx->m;
+	int arp;
+
+	if (HAS_QUIRK(QUIRK_FT2BUGS)) {
+		arp = ft2_arpeggio(ctx, xc);
+	} else {
+		arp = xc->arpeggio.val[xc->arpeggio.count];
+	}
+
+	xc->arpeggio.count++;
+	xc->arpeggio.count %= xc->arpeggio.size;
+
+	return arp;
+}
+
 static int is_first_frame(struct context_data *ctx)
 {
 	struct player_data *p = &ctx->p;
@@ -513,7 +528,12 @@ static inline void read_row(struct context_data *ctx, int pat, int row)
 		}
 
 		if (check_delay(ctx, &ev, chn) == 0) {
-			if (!f->rowdelay_set || f->rowdelay > 0) {
+			/* rowdelay_set bit 1 is set only in the first tick of the row
+			 * event if the delay causes the tick count resets to 0. We test
+			 * it to read row events only in the start of the row. (see the
+			 * OpenMPT test case FineVolColSlide.it)
+			 */
+			if (!f->rowdelay_set || ((f->rowdelay_set & ROWDELAY_FIRST_FRAME) && f->rowdelay > 0)) {
 				libxmp_read_event(ctx, &ev, chn);
 #ifndef LIBXMP_CORE_PLAYER
 				libxmp_med_hold_hack(ctx, pat, chn, row);
@@ -769,6 +789,7 @@ static void process_frequency(struct context_data *ctx, int chn, int act)
 	struct channel_data *xc = &p->xc_data[chn];
 	struct xmp_instrument *instrument;
 	double period, vibrato;
+	double final_period;
 	int linear_bend;
 	int frq_envelope;
 	int arp;
@@ -834,39 +855,22 @@ static void process_frequency(struct context_data *ctx, int chn, int act)
 	period += libxmp_extras_get_period(ctx, xc);
 #endif
 
+	if (HAS_QUIRK(QUIRK_ST3BUGS)) {
+		if (period < 0.25) {
+			libxmp_virt_resetchannel(ctx, chn);
+		}
+	}
 	/* Sanity check */
 	if (period < 0.1) {
 		period = 0.1;
 	} 
 
 	/* Arpeggio */
-
-	if (HAS_QUIRK(QUIRK_FT2BUGS)) {
-		arp = ft2_arpeggio(ctx, xc);
-	} else {
-		arp = xc->arpeggio.val[xc->arpeggio.count];
-	}
+	arp = arpeggio(ctx, xc);
 
 	/* Pitch bend */
 
- 	/* From OpenMPT PeriodLimit.s3m:
-	 * "ScreamTracker 3 limits the final output period to be at least 64,
-	 *  i.e. when playing a note that is too high or when sliding the
-	 *  period lower than 64, the output period will simply be clamped to
-	 *  64. However, when reaching a period of 0 through slides, the
-	 *  output on the channel should be stopped."
-	 */
-	/* ST3 uses periods*4, so the limit is 16. Adjusted to the exact
-	 * A6 value because we compute periods in floating point.
-	 */
-	if (HAS_QUIRK(QUIRK_ST3BUGS)) {
-		if (period < 16.239270) {	/* A6 */
-			period = 16.239270;
-		}
-	}
-
-	linear_bend = libxmp_period_to_bend(ctx, period + vibrato, xc->note,
-							xc->per_adj);
+	linear_bend = libxmp_period_to_bend(ctx, period + vibrato, xc->note, xc->per_adj);
 
 	if (TEST_NOTE(NOTE_GLISSANDO) && TEST(TONEPORTA)) {
 		if (linear_bend > 0) {
@@ -920,12 +924,29 @@ static void process_frequency(struct context_data *ctx, int chn, int act)
 	linear_bend += libxmp_extras_get_linear_bend(ctx, xc);
 #endif
 
-	period = libxmp_note_to_period_mix(xc->note, linear_bend);
-	libxmp_virt_setperiod(ctx, chn, period);
+	final_period = libxmp_note_to_period_mix(xc->note, linear_bend);
+
+	/* From OpenMPT PeriodLimit.s3m:
+	 * "ScreamTracker 3 limits the final output period to be at least 64,
+	 *  i.e. when playing a note that is too high or when sliding the
+	 *  period lower than 64, the output period will simply be clamped to
+	 *  64. However, when reaching a period of 0 through slides, the
+	 *  output on the channel should be stopped."
+	 */
+	/* ST3 uses periods*4, so the limit is 16. Adjusted to the exact
+	 * A6 value because we compute periods in floating point.
+	 */
+	if (HAS_QUIRK(QUIRK_ST3BUGS)) {
+		if (final_period < 16.239270) {	/* A6 */
+			final_period = 16.239270;
+		}
+	}
+
+	libxmp_virt_setperiod(ctx, chn, final_period);
 
 	/* For xmp_get_frame_info() */
 	xc->info_pitchbend = linear_bend >> 7;
-	xc->info_period = period * 4096;
+	xc->info_period = final_period * 4096;
 
 	if (IS_PERIOD_MODRNG()) {
 		CLAMP(xc->info_period,
@@ -1092,11 +1113,10 @@ static void update_volume(struct context_data *ctx, int chn)
 			 * ever executed on the first tick -- not on multiples
 			 * of the first tick if there is a pattern delay. 
 			 */
-			if (!f->rowdelay_set || f->rowdelay_set & 2) {
+			if (!f->rowdelay_set || f->rowdelay_set & ROWDELAY_FIRST_FRAME) {
 				xc->volume += xc->vol.fslide2;
 			}
 		}
-		f->rowdelay_set &= ~2;
 #endif
 
 		if (TEST(TRK_FVSLIDE)) {
@@ -1186,9 +1206,6 @@ static void update_frequency(struct context_data *ctx, int chn)
 		break;
 	}
 
-	xc->arpeggio.count++;
-	xc->arpeggio.count %= xc->arpeggio.size;
-
 	/* Check for invalid periods (from Toru Egashira's NSPmod)
 	 * panic.s3m has negative periods
 	 * ambio.it uses low (~8) period values
@@ -1293,14 +1310,14 @@ static void play_channel(struct context_data *ctx, int chn)
 
 	libxmp_virt_release(ctx, chn, TEST_NOTE(NOTE_RELEASE));
 
-	process_volume(ctx, chn, act);
-	process_frequency(ctx, chn, act);
-	process_pan(ctx, chn, act);
-
 	update_volume(ctx, chn);
 	update_frequency(ctx, chn);
 	update_pan(ctx, chn);
 
+	process_volume(ctx, chn, act);
+	process_frequency(ctx, chn, act);
+	process_pan(ctx, chn, act);
+
 #ifndef LIBXMP_CORE_PLAYER
 	if (HAS_QUIRK(QUIRK_PROTRACK) && xc->ins < mod->ins) {
 		update_invloop(m, xc);
@@ -1502,8 +1519,13 @@ int xmp_start_player(xmp_context opaque, int rate, int format)
 	p->loop_count = 0;
 	p->sequence = 0;
 
-	/* Unmute all channels and set default volume */
-	for (i = 0; i < XMP_MAX_CHANNELS; i++) {
+	/* Set default volume and mute status */
+	for (i = 0; i < mod->chn; i++) {
+		if (mod->xxc[i].flg & XMP_CHANNEL_MUTE)
+			p->channel_mute[i] = 1;
+		p->channel_vol[i] = 100;
+	}
+	for (i = mod->chn; i < XMP_MAX_CHANNELS; i++) {
 		p->channel_mute[i] = 0;
 		p->channel_vol[i] = 100;
 	}
@@ -1553,6 +1575,9 @@ int xmp_start_player(xmp_context opaque, int rate, int format)
 		goto err1;
 	}
 
+	/* Reset our buffer pointers */
+	xmp_play_buffer(opaque, NULL, 0, 0);
+
 #ifndef LIBXMP_CORE_PLAYER
 	for (i = 0; i < p->virt.virt_channels; i++) {
 		struct channel_data *xc = &p->xc_data[i];
@@ -1569,9 +1594,11 @@ int xmp_start_player(xmp_context opaque, int rate, int format)
 #ifndef LIBXMP_CORE_PLAYER
     err2:
 	free(p->xc_data);
+	p->xc_data = NULL;
 #endif
     err1:
 	free(f->loop);
+	f->loop = NULL;
     err:
 	return ret;
 }
@@ -1698,6 +1725,8 @@ int xmp_play_frame(xmp_context opaque)
 		play_channel(ctx, i);
 	}
 
+	f->rowdelay_set &= ~ROWDELAY_FIRST_FRAME;
+
 	p->frame_time = m->time_factor * m->rrate / p->bpm;
 	p->current_time += p->frame_time;
 
