@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2016 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2021 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -26,9 +26,6 @@
 
 #ifndef LIBXMP_CORE_PLAYER
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
 #include <limits.h>
 #include <unistd.h>
 
@@ -36,28 +33,44 @@
 #include <sys/stat.h>
 #endif
 
+#include "common.h" /* for libxmp_snprintf */
 #include "tempfile.h"
 
-#ifdef WIN32
+#ifdef _WIN32
 
 int mkstemp(char *);
 
-static int get_temp_dir(char *buf, int size)
+static int get_temp_dir(char *buf, size_t size)
 {
-	const char def[] = "C:\\WINDOWS\\TEMP";
-	char *tmp = getenv("TEMP");
-
-	strncpy(buf, tmp ? tmp : def, size);
-	strncat(buf, "\\", size);
-
+	static const char def[] = "C:\\WINDOWS\\TEMP";
+	const char *tmp = getenv("TEMP");
+	snprintf(buf, size, "%s\\", (tmp != NULL)? tmp : def);
 	return 0;
 }
 
-#elif defined __AMIGA__
+#elif defined(__OS2__)
 
-static int get_temp_dir(char *buf, int size)
+static int get_temp_dir(char *buf, size_t size)
 {
-	strncpy(buf, "T:", size);
+	static const char def[] = "C:";
+	const char *tmp = getenv("TMP");
+	snprintf(buf, size, "%s\\", (tmp != NULL)? tmp : def);
+	return 0;
+}
+
+#elif defined(__MSDOS__)
+
+static int get_temp_dir(char *buf, size_t size)
+{
+	strcpy(buf, "C:\\"); /* size-safe against PATH_MAX */
+	return 0;
+}
+
+#elif defined LIBXMP_AMIGA
+
+static int get_temp_dir(char *buf, size_t size)
+{
+	strcpy(buf, "T:"); /* size-safe against PATH_MAX */
 	return 0;
 }
 
@@ -66,7 +79,7 @@ static int get_temp_dir(char *buf, int size)
 #include <sys/types.h>
 #include <sys/stat.h>
 
-static int get_temp_dir(char *buf, int size)
+static int get_temp_dir(char *buf, size_t size)
 {
 #define APPDIR "/sdcard/Xmp for Android"
 	struct stat st;
@@ -83,11 +96,11 @@ static int get_temp_dir(char *buf, int size)
 	return 0;
 }
 
-#else
+#else /* unix */
 
-static int get_temp_dir(char *buf, int size)
+static int get_temp_dir(char *buf, size_t size)
 {
-	char *tmp = getenv("TMPDIR");
+	const char *tmp = getenv("TMPDIR");
 
 	if (tmp) {
 		snprintf(buf, size, "%s/", tmp);
