@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2016 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2021 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -20,8 +20,6 @@
  * THE SOFTWARE.
  */
 
-#include <stdlib.h>
-#include <string.h>
 #include <limits.h>
 #include "common.h"
 #include "virtual.h"
@@ -150,6 +148,7 @@ int libxmp_virt_on(struct context_data *ctx, int num)
 	}
 #endif
 	free(p->virt.voice_array);
+	p->virt.voice_array = NULL;
       err:
 	return -1;
 }
@@ -177,6 +176,8 @@ void libxmp_virt_off(struct context_data *ctx)
 
 	free(p->virt.voice_array);
 	free(p->virt.virt_channel);
+	p->virt.voice_array = NULL;
+	p->virt.virt_channel = NULL;
 }
 
 void libxmp_virt_reset(struct context_data *ctx)
@@ -428,7 +429,7 @@ void libxmp_virt_setnna(struct context_data *ctx, int chn, int nna)
 }
 
 static void check_dct(struct context_data *ctx, int i, int chn, int ins,
-			int smp, int note, int dct, int dca)
+			int smp, int note, int nna, int dct, int dca)
 {
 	struct player_data *p = &ctx->p;
 	struct mixer_voice *vi = &p->virt.voice_array[i];
@@ -437,13 +438,24 @@ static void check_dct(struct context_data *ctx, int i, int chn, int ins,
 	voc = p->virt.virt_channel[chn].map;
 
 	if (vi->root == chn && vi->ins == ins) {
-		if (dct == XMP_INST_DCT_INST ||
-			(dct == XMP_INST_DCT_SMP && vi->smp == smp) ||
-			(dct == XMP_INST_DCT_NOTE && vi->note == note)) {
 
-			if (dca) {
-				if (i != voc || vi->act)
+		if (nna == XMP_INST_NNA_CUT) {
+		    libxmp_virt_resetvoice(ctx, i, 1);
+		    return;
+		}
+
+		vi->act = nna;
+
+		if ((dct == XMP_INST_DCT_INST) ||
+		    (dct == XMP_INST_DCT_SMP && vi->smp == smp) ||
+		    (dct == XMP_INST_DCT_NOTE && vi->note == note)) {
+
+			if (nna == XMP_INST_NNA_OFF && dca == XMP_INST_DCA_FADE) {
+				vi->act = VIRT_ACTION_OFF;
+			} else if (dca) {
+				if (i != voc || vi->act) {
 					vi->act = dca;
+				}
 			} else {
 				libxmp_virt_resetvoice(ctx, i, 1);
 			}
@@ -485,7 +497,7 @@ int libxmp_virt_setpatch(struct context_data *ctx, int chn, int ins, int smp,
 		int i;
 
 		for (i = 0; i < p->virt.maxvoc; i++) {
-			check_dct(ctx, i, chn, ins, smp, note, dct, dca);
+			check_dct(ctx, i, chn, ins, smp, note, nna, dct, dca);
 		}
 	}
 #endif
