@@ -2,11 +2,11 @@
  * Win32 debug message helper by Mirko Buffoni
  */
 
-#if defined WIN32 && defined _DEBUG
+#if defined(_WIN32) && defined(_DEBUG)
 
-#include <stdio.h>
-#include <stdarg.h>
-void _D(const char *format, ...)
+#include "common.h"
+
+void D_(const char *format, ...)
 {
 	va_list argptr;
 
